# jsb/utils/textutils.py
#
#

## basic imports

import cgi
import re
import html.entities

## unescape_charref function

def unescape_charref(ref): 
    """ ask maze. """
    name = ref[2:-1] 
    base = 10 
    if name.startswith("x"): 
        name = name[1:] 
        base = 16 
    return chr(int(name, base)) 

## replace_entities function

def replace_entities(match):
    """ ask maze. """ 
    ent = match.group() 
    if ent[1] == "#":  return unescape_charref(ent) 
    repl = html.entities.najsbcodepoint.get(ent[1:-1]) 
    if repl is not None: repl = chr(repl) 
    else: repl = ent 
    return repl 

## html_unescape function

def html_unescape(data): 
    ''' unescape (numeric) HTML entities. '''
    return re.sub(r"&#?[A-Za-z0-9]+?;", replace_entities, data) 

## html_escape function

def html_escape(data):
    ''' escape HTML entities. '''
    return cgi.escape(data)



# jsb/plugs/core/misc.py
#
#

""" misc commands. """

## jsb imports

from ggz.utils.exception import handle_exception
from ggz.lib.commands import cmnds
from ggz.lib.examples import examples
from ggz.lib.persiststate import UserState

## basic imports

import time
import os
import threading
import _thread
import copy

## defines

cpy = copy.deepcopy

## test command

def handle_test(bot, ievent):
    """ no arguments - give test response. """
    ievent.reply("%s (%s) - %s - it works!" % (ievent.auth or ievent.userhost, ievent.nick, ievent.user.data.name))
    
cmnds.add('test', handle_test, ['USER', 'GUEST'])
examples.add('test', 'give test response',' test')

## source command

def handle_source(bot, ievent):
    """ no arguments - show where to fetch the bot source. """ 
    ievent.reply('see http://jsonbot.googlecode.com')

cmnds.add('source', handle_source, ['USER', 'GUEST'])
examples.add('source', 'show source url', 'source')



# jsb/plugs/common/koffie.py
#
# Made by WildRover and Junke1990

""" schenk wat koffie! """

## jsb imports

from ggz.lib.commands import cmnds
from ggz.lib.examples import examples

## basic imports

import os
import string
import random

## defines

koffie = []
thee = []
bier = []
wijn = []
fris = []
taart = []
koek = []
chips = []
soep = []
sex = []
roken = []
beledig = []

## init function

def init():
    global koffie
    global thee
    global bier
    global wijn
    global fris
    global taart
    global koek
    global chips
    global soep
    global sex
    global roken
    global beledig
    for i in  koffietxt.split('\n'):
        if i:
            koffie.append(i.strip())
    for i in theetxt.split('\n'):
        if i:
            thee.append(i.strip())
    for i in biertxt.split('\n'):
        if i:
            bier.append(i.strip())
    for i in wijntxt.split('\n'):
        if i:
            wijn.append(i.strip())
    for i in fristxt.split('\n'):
        if i:
            fris.append(i.strip())
    for i in taarttxt.split('\n'):
        if i:
            taart.append(i.strip())
    for i in koektxt.split('\n'):
        if i:
            koek.append(i.strip())
    for i in chipstxt.split('\n'):
        if i:
            chips.append(i.strip())
    for i in soeptxt.split('\n'):
        if i:
            soep.append(i.strip())
    for i in sextxt.split('\n'):
        if i:
            sex.append(i.strip())
    for i in rokentxt.split('\n'):
        if i:
            roken.append(i.strip())
    for i in beledigtxt.split('\n'):
        if i:
            beledig.append(i.strip())
    return 1

## functions

def do(bot, ievent, txt):
    if not bot.allowall: bot.action(ievent.channel, txt, event=ievent)
    else: bot.say(ievent.channel, txt, event=ievent)

## koffie command

def handle_koffie(bot, ievent):
    """ arguments: [<nick>] - get/give a coffee """
    rand = random.randint(0,len(koffie)-1)
    try:
        input = ievent.args[0]
        nick = '%s' % input
    except:
        if len('%s') >= 0: nick = ievent.nick
    do(bot, ievent, koffie[rand] + " " + nick)

cmnds.add('koffie', handle_koffie, 'USER')
examples.add('koffie', 'get a koffie quote', 'koffie')

## thee command

def handle_thee(bot, ievent):
    """ arguments: [<nick>] - get/give a thee """
    rand = random.randint(0,len(thee)-1)
    try:
        input = ievent.args[0]
        nick = '%s' % input
    except:
        if len('%s') >= 0: nick = ievent.nick
    do(bot, ievent, thee[rand] + " " + nick)

cmnds.add('thee', handle_thee, 'USER')
examples.add('thee', 'get an thee', 'thee')

## bier command

def handle_bier(bot, ievent):
    """ arguments: [<nick>] - get a beer  """
    rand = random.randint(0,len(bier)-1)
    try:
        input = ievent.args[0]
        nick = '%s' % input
    except:
        if len('%s') >= 0: nick = ievent.nick
    do(bot, ievent, bier[rand] + " " + nick)

cmnds.add('bier', handle_bier, 'USER')
examples.add('bier', 'get a bier', 'bier')

## wijn command

def handle_wijn(bot, ievent):
    """ arguments: [<nick>] - get a wine  """
    rand = random.randint(0,len(wijn)-1)
    try:
        input = ievent.args[0]   
        nick = '%s' % input
    except:
        if len('%s') >= 0: nick = ievent.nick
    do(bot, ievent, wijn[rand] + " " + nick)

cmnds.add('wijn', handle_wijn, 'USER')
examples.add('wijn', 'get a wijn', 'wijn')

## fris command

def handle_fris(bot, ievent):
    """ arguments: [<nick>] - get a fris  """
    rand = random.randint(0,len(fris)-1)
    try:
        input = ievent.args[0]
        nick = '%s' % input
    except:
        if len('%s') >= 0: nick = ievent.nick
    do(bot, ievent, fris[rand] + " " + nick)

cmnds.add('fris', handle_fris, 'USER')
examples.add('fris', 'get a fris', 'fris')

## taart command

def handle_taart(bot, ievent):
    """ arguments: [<nick>] - get a taart  """
    rand = random.randint(0,len(taart)-1)
    try:
        input = ievent.args[0]
        nick = '%s' % input
    except:
        if len('%s') >= 0: nick = ievent.nick
    do(bot, ievent, taart[rand] + " " + nick)

cmnds.add('taart', handle_taart, 'USER')
examples.add('taart', 'get a taart', 'taart')

## koek command

def handle_koek(bot, ievent):
    """ arguments: [<nick>] - get a koek  """
    rand = random.randint(0,len(koek)-1)
    try:
        input = ievent.args[0]
        nick = '%s' % input
    except:
        if len('%s') >= 0: nick = ievent.nick
    do(bot, ievent, koek[rand] + " " + nick)

cmnds.add('koek', handle_koek, 'USER')
examples.add('koek', 'get a koek', 'koek')

cmnds.add('koekje', handle_koek, 'USER')
examples.add('koekje', 'get a koekje', 'koekje')

## chips command

def handle_chips(bot, ievent):
    """ arguments: [<nick>] - get a chips  """
    rand = random.randint(0,len(chips)-1)
    try:
        input = ievent.args[0]
        nick = '%s' % input
    except:
        if len('%s') >= 0: nick = ievent.nick
    do(bot, ievent, chips[rand] + " " + nick)

cmnds.add('chips', handle_chips, 'USER')
examples.add('chips', 'get a chips', 'chips')

## soep command

def handle_soep(bot, ievent):
    """ arguments: [<nick>] - get a soep  """
    rand = random.randint(0,len(soep)-1)
    try:
        input = ievent.args[0]
        nick = '%s' % input
    except:
        if len('%s') >= 0: nick = ievent.nick
    do(bot, ievent, soep[rand] + " " + nick)

cmnds.add('soep', handle_soep, 'USER')
examples.add('soep', 'get a soep', 'soep')

## sex command

def handle_sex(bot, ievent):
    """ arguments: [<nick>] - get a sex  """
    rand = random.randint(0,len(sex)-1)
    try:
        input = ievent.args[0]
        nick = '%s' % input
    except:
        if len('%s') >= 0: nick = ievent.nick
    do(bot, ievent, sex[rand] + " " + nick)

cmnds.add('sex', handle_sex, 'USER')
examples.add('sex', 'get a sex', 'sex')

## roken command

def handle_roken(bot, ievent):
    """ arguments: [<nick>] - get a roken  """
    rand = random.randint(0,len(roken)-1)
    try:
        input = ievent.args[0]
        nick = '%s' % input
    except:
        if len('%s') >= 0: nick = ievent.nick
    do(bot, ievent, roken[rand] + " " + nick)

cmnds.add('roken', handle_roken, 'USER')
examples.add('roken', 'get a roken', 'roken')

## beledig command

def handle_beledig(bot, ievent):
    """ arguments: [<nick>] - get/give an belediging  """
    rand = random.randint(0,len(beledig)-1)
    try:
        input = ievent.args[0]
        nick = '%s' % input
    except:
        if len('%s') >= 0: nick = ievent.nick
    do(bot, ievent, beledig[rand] + " " + nick)

cmnds.add('beledig', handle_beledig, 'USER')
examples.add('beledig', 'get/give an belediging ', 'beledig')

## text definitions

koffietxt = """ schenkt een kopje koffie in voor
schenkt een kopje koffie wiener melange in voor
schenkt een kopje espresso voor je in voor
schenkt een kopje arabica voor je in voor
schenkt je een kopje koffie verkeerd in voor
schenkt een kopje cappuccino in voor
schenkt een kopje cafe choco in voor
schenkt je een kopje koffie van gisteren in voor
vult een kopje met decafe voor
schenkt een kopje toebroek in voor
maak het gerust zelf,
geeft een glas irish-coffee aan
geeft een glas french-coffee aan
schenkt een kopje turkse koffie in voor
schenkt een kopje americano in voor
schenkt een kopje doppio in voor
schenkt een kopje espresso ristretto in voor
schenkt een kopje caffe latte in voor
schenkt een kopje espresso macchiato in voor
schenkt een kopje latte macchiato in voor
schenkt een kopje espresso romano in voor
schenkt een kopje con panna voor in voor
schenkt een kopje mocha in voor
"""

theetxt = """ maak het gerust zelf,
schenkt een kopje groene thee in voor
ja lekker heh, 
schenkt een kopje mintthee in voor 
schenkt een kopje brandnetelthee in voor 
schenkt een kopje bosvruchtenthee in voor
schenkt een kopje citroenthee in voor
schenkt een kopje rooibosthee in voor
schenkt een heerlijk kopje lavendelthee in voor
maakt een kopje fruity forest voor
maakt een kopje tempting red voor
maakt een kopje early grey voor
"""

biertxt = """ reikt een flesje hertog-jan aan voor
tapt een glas witbier voor
geeft een flesje amstel aan
geeft een flesje jupiler aan
geeft een flesje bavaria aan
geeft een flesje grolsch aan
pak het gerust zelf,
geeft een flesje hoegaarden aan
geeft een flesje grimbergen aan
reikt een flesje dommelsch aan voor 
reikt een flesje brand aan voor
schenkt een glas duvel in voor
schenkt een glas chouffe in voor
"""

wijntxt = """ trekt een fles rode wijn open.
geeft een glas rode wijn aan
geeft een glas witte wijn aan
geeft een glas rose aan
geeft een glas champagne aan
trekt een fles appelwijn open, tast toe,
geeft een glas cider aan
geeft een glas champagne aan
pak het gerust zelf maar,
morst rode wijn over je witte shirt, oops... sorry,
ja lekker,
witte, rode of rose, 
geeft een glas droge witte wijn aan
geeft een glas zoete witte wijn aan
trekt de kurk van een fles witte wijn, zuipt hem helemaal leeg *hips* en proost,
schenkt een glas goedkope tafelwijn in voor
een glas chateaux-migraine voor
trekt een fles hema-huiswijn open en schenkt vast een kom in voor
"""

fristxt = """ zet de deur en het raam open voor
geeft een glas cola aan
geeft een glas cola light aan
geeft een glas cherry coke aan
geeft een glas pepsi max aan
geeft een glas sinas aan
geeft een glas fanta aan
geeft een glas sprite aan
geeft een glas 7-up aan
pakt een flesje chocomel voor
geeft een glas jus d'orange aan
schenkt een glas appelsap in voor
sorry, het is te koud voor fris,
wijst naar een lege koelkast. sorry
ja lekker, doe mij ook wat,
"""

taarttxt = """ geeft je een punt appletaart met lekker veel slagroom.
bak het gerust zelf, alles ligt klaar voor je,
geeft een taart met kaas, salami, oregano en knoflook aan
geeft een punt chocoladetaart aan
geeft een punt slagroomtaart aan
geeft een punt appeltaart aan
geeft een punt aardbeientaart aan
geeft een punt vruchtentaart aan
geeft een punt advocaattaart aan
geeft een punt abrikozentaart aan
geeft een punt appelcitroentaart aan
geeft een punt appelkruimeltaart aan
geeft een punt bananentaart aan
geeft een punt boerenjongenstaart aan
geeft een punt bosvruchtentaart aan
geeft een punt champagnetaart aan
geeft een punt kersentaart aan
geeft een punt rijsttaart aan
geeft een punt mokkataart aan
geeft een punt notentaart aan
geeft een punt peche-melbataart aan
geeft een punt perentaart aan
geeft een punt schwarzwaldertaart aan
geeft een punt stroopwafeltaart aan
geeft een punt tiramisutaart aan
geeft een punt yoghurt-sinaastaart aan
geeft een punt zwitserse-roomtaart aan
"""

koektxt = """ geeft een lange vinger aan
geeft een speculaasje aan
geeft een stroopwafel aan
geeft een zandkoekje aan
geeft een sprits aan
geeft een muffin aan
geeft een slagroomsoesje aan
geeft een walnootkoekje aan
geeft een lekker kaaskoekje aan
geeft een honingkoekje aan
geeft een hazelnootkoekje aan
geeft een gemberkoekje aan
geeft een eclair aan
geeft een cremerolletje aan
geeft een chocoladekoekje aan
geeft een pindarotsje aan
geeft een bokkenpootje aan
geeft een lebkuchen aan
geeft een amandelkoekje aan
bak ze gerust zelf,
"""

chipstxt = """ geeft je een bakje bugles.
geeft een zakje naturel chips aan
pakt een zakje wokkels voor
geeft een zakje paprika chips aan
geeft een zakje bolognese chips aan
geeft een zakje barbecue ham chips aan
geeft een zakje cheese onion aan
geeft een zakje mexican salsa aan
pakt een zakje bugles voor
geeft een zakje hamka's.
*crunch* dank je,
loert tevergeefs in een lege chipszak voor
ja lekker,
"""

soeptxt = """ geeft een kom aspergesoep aan
geeft een kom bospaddestoelensoep aan
geeft een kom frisse erwtensoep aan
geeft een kom kip tikka soep aan
geeft een kom kruidige tomatensoep aan
geeft een kom mosterdsoep aan
geeft een kom ossenstaartsoep aan
geeft een kom pittige tomatensoep aan
geeft een kom romige kippensoep aan
geeft een kom romige tomatensoep aan
geeft een bord boerengroentesoep aan
geeft een kom champignonsoep aan
geeft een kom erwtensoep aan
geeft een kom goulashsoep aan
geeft een kom kippensoep aan
geeft een kom tomaten-creme soep aan
geeft een kom tomatensoep aan
geeft een kom bruine bonensoep aan
geeft een kom chinese tomatensoep aan
geeft een kom groentesoep aan
geeft een kom tomaten-groentesoep aan
geeft een kom uiensoep aan
geeft een kom kerrie-cremesoep aan
maak voor ons ook wat,
staart in een lege pan voor
vist een ouwe schoen uit de bouillon voor
"""

sextxt = """ huh? moeilijk? waar?
Snufj!
Nee nu niet, ik heb hoofdpijn.
Wacht dan doe ik een condoom aan.
Veel te koud voor sex.
Veel te warm voor sex.
Ja strakjes, eerst nog even dit afmaken...
Ja lekker!
Moo!
Heb je de sleutel van m'n kuisheidsgordel dan?
Rook jij ook na de sex?
Mag 't licht uit?
"""

rokentxt = """ Er is maar 1 weg na de longen en die moet geasfalteerd worden
Ohhh!!! heerlijk!!!.....
Riet rokers sterven ook!
Het nieuwe lied van K3: Rokus Spokus, iedereen kan roken!
Elk pakje sigaretten verkort je leven met 11 minuten, elke vrijjpartij verlengt je leven met 15 minuten. conclusie: ROKERS NEUK VOOR JE LEVEN!
Niet zeiken lekker ding, ik betaal jouw belasting!
Afspraak: Rook jij mijn laatste, koop jij een nieuwe.
Als mijn moeder dit pakje vind is het van jou!
Roken werkt ontspannend en is dus goed tegen hartklachten en algemene  stress.
Ex-rokers moeten helemaal hun bek houden!
Niet roken bespaard geld? waarom heb jij t dan nooit?
Koop je eigen pakje KLOOTZAK!
Ik rook dus ik ben rijk, neuken?
Ik geef om jouw gezondheid dus blijf godverdomme van mijn peuken af!
Roken is slecht voor de zwangerschap, dat condoom kun je dus in je zak houden.
Roken is ongezond..... een dikke vatsige vetklep ook!
Roken na de sex houd mijn gezondheid in balans.
Moet ik er een bewaren voor na de sex of rook je niet?
Heb je last van je keel door mijn gerook? moet je er ook een opsteken, ik heb nooit last!
Wie meerookt hoort ook me te betalen!
Stoppen? Ik probeer het zo'n 20 keer per dag!
"""

beledigtxt = """schopt
misbruikt
verhuurt
slaat
mept
gooit koffie over
bitchslapt
schiet op
schijt in de mond van
vist in het aquarium van
pakt een koekenpan en slaat daarmee
hackt
stampt zo hard tusse de noten dat je je kindergeld kwijt bent, 
moont
castreert
verloot
"""



import sys
import hmac

from sleekxmpp.thirdparty.suelta.util import hash, bytes
from sleekxmpp.thirdparty.suelta.sasl import Mechanism, register_mechanism
from sleekxmpp.thirdparty.suelta.exceptions import SASLError, SASLCancelled


class CRAM_MD5(Mechanism):

    """
    """

    def __init__(self, sasl, name):
        """
        """
        super(CRAM_MD5, self).__init__(sasl, name, 2)

        self.hash = hash(name[5:])
        if self.hash is None:
            raise SASLCancelled(self.sasl, self)
        if not self.sasl.tls_active():
            if not self.sasl.sec_query(self, 'CRAM-MD5'):
                raise SASLCancelled(self.sasl, self)

    def prep(self):
        """
        """
        if 'savepass' not in self.values:
            if self.sasl.sec_query(self, 'CLEAR-PASSWORD'):
                self.values['savepass'] = True

        if 'savepass' not in self.values:
            del self.values['password']

    def process(self, challenge):
        """
        """
        if challenge is None:
            return None

        self.check_values(['username', 'password'])
        username = bytes(self.values['username'])
        password = bytes(self.values['password'])

        mac = hmac.HMAC(key=password, digestmod=self.hash)

        mac.update(challenge)

        return username + b' ' + bytes(mac.hexdigest())

    def okay(self):
        """
        """
        return True

    def get_user(self):
        """
        """
        return self.values['username']


register_mechanism('CRAM-', 20, CRAM_MD5)



# jsb/plugs/common/tour.py
#
#

""" do a tour of the bot. """

eventlist = ["!welcome", "GGZBOT provides functionality through the use of plugins, you can use the !list comamnd to see what plugins are available =>", "!list"]

## jsb imports

from ggz.lib.commands import cmnds
from ggz.lib.examples import examples

## basic imports

import time

## dotour command

def handle_dotour(bot, event):
    if event.user.state.data.notour: event.reply("the tour is disabled for %s" % event.userhost) ; return
    event.reply("will say something every 5 seconds. you can disable this tour by typing !set notour 1")
    time.sleep(5)
    for txt in eventlist:
        if event.user.state.data.notour: break 
        if txt.startswith("!"):
            e = bot.make_event(event.userhost, event.channel, txt[1:], 0, event)
            e.execute()
        else: event.reply(txt)
        time.sleep(5)
    event.done()

cmnds.add("tour", handle_dotour, ["OPER", "USER", "GUEST"], threaded=True)
examples.add("tour", "show a tour of the bot", "tour")


# jsb/reboot.py
#
#

""" reboot code. """

## jsb imports

from ggz.lib.fleet import getfleet
from ggz.imports import getjson
json = getjson()

## basic imports

import os
import sys
import pickle
import tempfile
import logging
import time

## reboot function

def reboot():
    """ reboot the bot. """
    logging.warn("reboot - rebooting")
    os.execl(sys.argv[0], *sys.argv)

## reboot_stateful function

def reboot_stateful(bot, ievent, fleet, partyline):
    """ reboot the bot, but keep the connections (IRC only). """
    logging.warn("reboot - doing statefull reboot")
    session = {'bots': {}, 'name': bot.cfg.name, 'channel': ievent.channel, 'partyline': []}
    fleet = getfleet()
    for i in fleet.bots:
        logging.warn("reboot - updating %s" % i.cfg.name)
        data = i._resumedata()
        if not data: continue
        session['bots'].update(data)
        if i.type == "sxmpp": i.exit() ; continue
        if i.type == "convore": i.exit() ; continue
        if i.type == "tornado":
            i.exit()
            time.sleep(0.1)
            for socketlist in list(i.websockets.values()):
                for sock in socketlist: sock.stream.close()
    session['partyline'] = partyline._resumedata()
    sfile, sessionfile = tempfile.mkstemp('-session', 'jsb-', text=True)
    logging.warn("writing session file %s" % sessionfile)
    json.dump(session, open(sessionfile, "w"))
    args = []
    skip = False
    for a in sys.argv[1:]:
        if skip: skip = False ; continue
        if a == "-r": skip = True ; continue
        args.append(a)
    os.execl(sys.argv[0], sys.argv[0], '-r', sessionfile, *args)



# jsb/plugs/common/gcalc.py
# encoding: utf-8
#
#

""" use google to calculate e.g. !gcalc 1 + 1 """

## jsb imports

from ggz.lib.commands import cmnds
from ggz.lib.examples import examples
from ggz.utils.url import useragent

## basic imports

import urllib.request, urllib.error, urllib.parse

## gcalc command

def handle_gcalc(bot, ievent):
    """ arguments: <expression> - use google calc. """
    if len(ievent.args) > 0: expr = " ".join(ievent.args).replace("+", "%2B").replace(" ", "+")
    else: ievent.missing('Missing an expression') ; return
    req = urllib.request.Request("http://www.google.com/ig/calculator?hl=en&q=%s" % expr, None,  {'User-agent': useragent()})
    data = urllib.request.urlopen(req).read()
    try:
        rhs = data.split("rhs")[1].split("\"")[1]
        lhs = data.split("lhs")[1].split("\"")[1]
        if rhs and lhs:
            ievent.reply("%s = %s" % (lhs,rhs.replace('\\x26#215;', '*').replace('\\x3csup\\x3e', '**').replace('\\x3c/sup\\x3e', '')))
        else: ievent.reply("hmmm can't get a result ..")
    except Exception as ex:
        ievent.reply(str(ex))    

cmnds.add('gcalc', handle_gcalc, ['OPER', 'USER', 'GUEST'])
examples.add('gcalc', 'calculate an expression using the google calculator', 'gcalc 1 + 1')



# jsb/plugs/core/cfg.py
#
#

""" this plugin manages various configuration settings. """

from ggz.lib.commands import cmnds
from ggz.lib.users import getusers
from ggz.lib.config import getmainconfig
from ggz.lib.aliases import setalias

allowset = ["timesleep", "dbenable"]

def handle_cfgset(bot, event):
    if len(event.args) != 3: event.missing("<configname> <variable> <value>") ; return
    name, var, value = event.args
    if not var in allowset: event.reply("setting %s is not allowed" % var) ; return
    if name == "main":
        if not getusers().allowed(event.userhost, "OPER"): event.reply("you need to have OPER permissions to edit the mainconfig.") ; return
        mcfg = getmainconfig()
        try: mcfg[var] = int(value)
        except ValueError: mcfg[var] = value
        mcfg.save()
        event.done()
    else: event.reply('we current only support editing the "main" config.') ; return

cmnds.add("cfg-set", handle_cfgset, ["OPER", "USER"])
setalias("mcfg-set", "cfg-set main")

def handle_cfg(bot, event):
    if len(event.args) != 1: event.missing("<configname>") ; return
    name = event.args[0]
    if name == "main":
        if not getusers().allowed(event.userhost, "OPER"): event.reply("you need to have OPER permissions to edit the mainconfig.") ; return
        event.reply(getmainconfig().fordisplay())

cmnds.add("cfg", handle_cfg, ["OPER", "USER"])
setalias("mcfg", "cfg main")



"""
    SleekXMPP: The Sleek XMPP Library
    Copyright (C) 2011  Nathanael C. Fritz
    This file is part of SleekXMPP.

    See the file LICENSE for copying permission.
"""

from sleekxmpp.features.feature_session.session import feature_session
from sleekxmpp.features.feature_session.stanza import Session



"""
    SleekXMPP: The Sleek XMPP Library
    Copyright (C) 2010 Nathanael C. Fritz, Lance J.T. Stout
    This file is part of SleekXMPP.

    See the file LICENSE for copying permission.
"""

from sleekxmpp.plugins.xep_0030 import stanza
from sleekxmpp.plugins.xep_0030.stanza import DiscoInfo, DiscoItems
from sleekxmpp.plugins.xep_0030.static import StaticDisco
from sleekxmpp.plugins.xep_0030.disco import xep_0030



# jsb/commands.py
#
#

""" 
    the commands module provides the infrastructure to dispatch commands. 
    commands are the first word of a line. 

"""

## jsb imports

from .threads import start_new_thread, start_bot_command
from ggz.utils.xmpp import stripped
from ggz.utils.trace import calledfrom, whichmodule
from ggz.utils.exception import handle_exception
from ggz.utils.lazydict import LazyDict
from .errors import NoSuchCommand, NoSuchUser
from .persiststate import UserState
from .runner import cmndrunner
from .boot import getcmndperms
from .floodcontrol import floodcontrol
from .aliases import getaliases, aliascheck

## basic imports

import logging
import sys
import types
import os
import copy
import time
import re

## defines

cpy = copy.deepcopy

## Command class

class Command(LazyDict):

    """ a command object. """

    def __init__(self, modname, cmnd, func, perms=[], threaded=False, wait=False, orig=None, how=None, speed=None):
        LazyDict.__init__(self)
        if not modname: raise Exception("modname is not set - %s" % cmnd)
        self.modname = cpy(modname)
        self.plugname = self.modname.split('.')[-1]
        self.cmnd = cpy(cmnd)
        self.orig = cpy(orig)
        self.func = func
        if type(perms) == bytes: perms = [perms, ]
        self.perms = cpy(perms)
        self.plugin = self.plugname
        self.threaded = cpy(threaded)
        self.wait = cpy(wait)
        self.enable = True
        self.how = how or "overwrite"
        self.regex = None
        self.speed = speed

class Commands(LazyDict):

    """
        the commands object holds all commands of the bot.
 
    """

    regex = []

    def add(self, cmnd, func, perms, threaded=False, wait=False, orig=None, how=None, speed=None, regex=False, *args, **kwargs):
        """ add a command. """
        modname = calledfrom(sys._getframe())
        try: prev = self[cmnd]
        except KeyError: prev = None
        target = Command(modname, cmnd, func, perms, threaded, wait, orig, how, speed=speed)
        if regex:
            logging.info("regex command detected - %s" % cmnd)
            self.regex.append(target)
            target.regex = cmnd 
            return self
        self[cmnd] = target
        try:
            p = cmnd.split('-')[0]
            if not self.pre: self.pre = LazyDict()
            if p in self.pre:
                if not self.pre[p]: self.pre[p] = []
                if prev in self.pre[p]: self.pre[p].remove(prev) 
                if target not in self.pre[p]: self.pre[p].append(target)
            else: self.pre[p] = [target, ]
        except IndexError: pass
        return self

    def checkre(self, bot, event):
        for r in self.regex:
            s = re.search(r.cmnd, event.stripcc().strip())
            if s:
                logging.info("regex matches %s" % r.cmnd)
                event.groups = list(s.groups())
                return r

    def wouldmatchre(self, bot, event, cmnd=""):
        groups = self.checkre(bot, event)
        if groups: return group
        
    def woulddispatch(self, bot, event):
        """ 
            dispatch an event if cmnd exists and user is allowed to exec this 
            command.

        """
        event.bind(bot)
        try:
            cmnd = event.stripcc().split()[0]
            if event.execstr and not cmnd: cmnd = event.execstr.split()[0]
            if not cmnd: cmnd = event.txt.split()[0]
        except Exception as ex: logging.debug("can't determine command") ; return None
        try:
            a = event.chan.data.aliases[cmnd]
            if a: cmnd = a.split()[0]
        except (KeyError, TypeError):
            try:
                a = getaliases()[cmnd]
                if a: cmnd = a.split()[0]
            except (KeyError, TypeError): 
                if cmnd not in self:
                    try:
                        from .boot import shorttable
                        if cmnd in shorttable.data:
                            cmndlist = shorttable.data[cmnd]
                            if len(cmndlist) == 1: cmnd = cmndlist[0]
                            else: event.reply("choose one of: ", cmndlist) ; return
                    except Exception as ex: handle_exception()
        logging.info("trying for %s" % cmnd)
        result = None
        try:
            result = self[cmnd]
        except KeyError: pass
        logging.debug("woulddispatch result: %s" % result)
        if result: event.bloh() ; event.makeargs()
        return result

    def dispatch(self, bot, event, direct=False):
        """ 
            dispatch an event if cmnd exists and user is allowed to exec this 
            command.

        """
        if event.nodispatch: logging.info("nodispatch is set on event") ; return
        if event.groupchat and bot.cfg.fulljids: id = event.auth
        elif event.groupchat: id = event.auth = event.userhost
        else: id = event.auth
        if not event.user: event.bind(bot)
        if not event.user: raise NoSuchUser(event.userhost)
        self.reloadcheck(bot, event)
        c = self.woulddispatch(bot, event)
        if not c: c = self.checkre(bot, event)
        if not c: raise NoSuchCommand(event.usercmnd)
        if c.modname in bot.plugs.loading and bot.plugs.loading[c.modname]: event.reply("%s is loading" % c.modname) ; return
        if bot.cmndperms and bot.cmndperms[c.cmnd]: perms = bot.cmndperms[c.cmnd]
        else: perms = c.perms
        if bot.allowall: return self.doit(bot, event, c, direct)
        elif not bot.users or bot.users.allowed(id, perms, bot=bot): return self.doit(bot, event, c, direct)
        elif bot.users.allowed(id, perms, bot=bot): return self.doit(bot, event, c, direct)
        return event

    def doit(self, bot, event, target, direct=False):
        """ do the dispatching. """
        if not target.enable: return
        if target.modname in event.chan.data.denyplug:
             logging.warn("%s is denied in channel %s - %s" % (target.plugname, event.channel, event.userhost))
             return
        id = event.auth or event.userhost
        event.iscommand = True
        event.how = event.how or target.how or "overwrite"
        event.thecommand = target
        aliascheck(event)
        logging.warning('dispatching %s (%s)' % (event.usercmnd, bot.cfg.name))
        try:
            if direct or event.direct: target.func(bot, event)
            elif target.threaded and not event.nothreads:
                logging.warning("launching thread for %s (%s)" % (event.usercmnd, bot.cfg.name))
                t = start_bot_command(target.func, (bot, event))
                event.thread = t
            else: event.dontclose = False; cmndrunner.put(target.speed or event.speed, target.modname, target.func, bot, event)
        except Exception as ex:
            logging.error('%s - error executing %s' % (whichmodule(), str(target.func)))
            raise
        return event

    def unload(self, modname):
        """ remove modname registered commands from store. """
        delete = []
        for name, cmnd in self.items():
            if not cmnd: continue
            if cmnd.modname == modname: delete.append(cmnd)
        for cmnd in delete: cmnd.enable = False
        return self

    def apropos(self, search):
        """ search existing commands for search term. """
        result = []
        from .boot import getcmndtable
        for name, plugname in getcmndtable().items():
            if search in name: result.append(name)
        return result

    def perms(self, cmnd):
        """ show what permissions are needed to execute cmnd. """
        try: return self[cmnd].perms
        except KeyError: return []

    def whereis(self, cmnd):
        """ return plugin name in which command is implemented. """
        from .boot import getcmndtable
        try: return getcmndtable()[cmnd]
        except KeyError: return ""

    def gethelp(self, cmnd):
        """ get the docstring of a command. used for help. """
        try: return self[cmnd].func.__doc__
        except KeyError: pass

    def reloadcheck(self, bot, event, target=None):
        """
            check if event requires a plugin to be reloaded. if so 
            reload the plugin.  

        """
        from .boot import getcmndtable
        from .boot import plugblacklist
        plugloaded = None
        plugin = None
        target = target or event.usercmnd.lower()
        from ggz.lib.aliases import getaliases
        aliases = getaliases()
        try: target = aliases[target]
        except KeyError:
            try: target = event.chan.data.aliases[target]
            except (AttributeError, KeyError, TypeError): pass
            if target not in getcmndtable():
                try:
                    from .boot import shorttable
                    if target in shorttable.data:
                        cmndlist = shorttable.data[target]
                        if len(cmndlist) == 1: target = cmndlist[0]
                except Exception as ex: handle_exception()
        if target: target = target.split()[0]
        logging.debug("checking for reload of %s" % target)
        try:
            plugin = getcmndtable()[target]
        except KeyError:
            try:
                from .boot import retable
                for regex, mod in retable.data.items():
                    if re.search(regex, event.stripcc() or event.txt): plugin = mod ; break
            except Exception as ex: handle_exception()
        logging.info("plugin is %s" % plugin)
        if not plugin: logging.debug("can't find plugin to reload for %s" % target) ; return
        if plugin in bot.plugs: logging.info(" %s already loaded" % plugin) ; return plugloaded
        elif plugin in plugblacklist.data: return plugloaded
        elif bot.cfg.loadlist and plugin not in bot.cfg.loadlist: logging.warn("plugin %s is blacklisted" % plugin) ; return plugloaded
        logging.info("loaded %s on demand" % plugin)
        plugloaded = bot.plugs.reload(plugin)
        return plugloaded

## global commands

cmnds = Commands()

def size():
    return len(cmnds)



# jsb/datadir.py
#
#

""" the data directory of the bot. """

## jsb imports

from ggz.utils.source import getsource

## basic imports

import re
import os
import shutil
import logging
import os.path
import getpass

## the global datadir

try: homedir = os.path.abspath(os.path.expanduser("~"))
except: homedir = os.getcwd()

datadir = homedir + os.sep + ".ggz"

## helper functions

def touch(fname):
    """ touch a file. """
    fd = os.open(fname, os.O_WRONLY | os.O_CREAT)
    os.close(fd)

def doit(ddir, mod, target=None):
    source = getsource(mod)
    if not source: raise Exception("can't find %s package" % mod)
    shutil.copytree(source, ddir + os.sep + (target or mod.replace(".", os.sep)))


## makedir function

def makedirs(ddir=None):
    """ make subdirs in datadir. """
    global datadir
    datadir = ddir or getdatadir()
    logging.warn("datadir - set to %s" % datadir)
    if not os.path.isdir(ddir):
        try: os.mkdir(ddir)
        except:
             raise Exception("can't make %s dir" % ddir)
        logging.info("making dirs in %s" % ddir)
    try: os.chmod(ddir, 0o700)
    except: pass
    if ddir: setdatadir(ddir)
    last = datadir.split(os.sep)[-1]
    if not os.path.exists(ddir + os.sep + "__init__.py"):
        try: touch(ddir + os.sep + "__init__.py")
        except: pass
    try:
        os.mkdir(ddir + os.sep + "config")
        touch(ddir + os.sep + "config" + os.sep + "__init__.py")
    except: pass
    # myplugs
    initsource = getsource("ggz.plugs")
    if not initsource: raise Exception("can't find ggz.plugs package")
    initsource = initsource + os.sep + "__init__.py"
    if not os.path.isdir(ddir + os.sep + 'myplugs'):
        os.mkdir(ddir + os.sep + 'myplugs')
        try: shutil.copy(initsource, ddir + os.sep + "myplugs" + os.sep + "__init__.py")
        except: pass
    # ggz-myplugs
    if os.path.isdir('ggz'):
        if not os.path.isdir('ggz-myplugs'):
            os.mkdir('ggz-myplugs')
            try: shutil.copy(initsource, "ggz-myplugs" + os.sep + "__init__.py")
            except: pass
    if not os.path.isdir('ggz-botlogs'): os.mkdir('ggz-botlogs')
    if not os.path.isdir(ddir + '/run/'): os.mkdir(ddir + '/run/')
    if not os.path.isdir(ddir + '/users/'): os.mkdir(ddir + '/users/')
    if not os.path.isdir(ddir + '/channels/'): os.mkdir(ddir + '/channels/')
    if not os.path.isdir(ddir + '/fleet/'): os.mkdir(ddir + '/fleet/')
    if not os.path.isdir(ddir + '/pgp/'): os.mkdir(ddir + '/pgp/')
    if not os.path.isdir(ddir + '/plugs/'): os.mkdir(ddir + '/plugs/')
    if not os.path.isdir(ddir + '/old/'): os.mkdir(ddir + '/old/')
    if not os.path.isdir(ddir + '/containers/'): os.mkdir(ddir + '/containers/')
    if not os.path.isdir(ddir + '/chatlogs/'): os.mkdir(ddir + '/chatlogs/')
    if not os.path.isdir(ddir + '/botlogs/'): os.mkdir(ddir + '/botlogs/')
    if not os.path.isdir(ddir + '/spider/'): os.mkdir(ddir + '/spider/')
    if not os.path.isdir(ddir + '/spider/data/'): os.mkdir(ddir + '/spider/data')
    if os.path.isfile(ddir + '/globals'):
        try: os.rename(ddir + '/globals', ddir + '/globals.old')
        except: pass
    if not os.path.isdir(ddir + '/globals/'): os.mkdir(ddir + '/globals/')

def getdatadir():
    global datadir
    return datadir

def setdatadir(ddir):
    global datadir
    datadir = ddir



# jsb/lib/config.py
#
#

""" config module. config is stored as item = JSON pairs. """

## jsb imports

from ggz.utils.trace import whichmodule, calledfrom
from ggz.utils.lazydict import LazyDict
from ggz.utils.exception import handle_exception
from ggz.utils.name import stripname
from .datadir import getdatadir
from .errors import CantSaveConfig, NoSuchFile
from ggz.utils.locking import lockdec

## simplejson imports

from ggz.imports import getjson
json = getjson()

## basic imports

import sys
import os
import types
import _thread
import logging
import uuid
import _thread
import getpass
import copy
import time

## locks

savelock = _thread.allocate_lock()
savelocked = lockdec(savelock)

## defines

cpy = copy.deepcopy

## classes

class Config(LazyDict):

    """ 
        config class is a dict containing json strings. is writable to file 
        and human editable.

    """

    def __init__(self, filename, verbose=False, input={}, ddir=None, nolog=False, *args, **kw):
        assert filename
        LazyDict.__init__(self, input, *args, **kw)
        self.origname = filename
        self.origdir = ddir or getdatadir()
        self.setcfile(ddir, filename)
        self.jsondb = None
        if not self._comments: self._comments = {}
        logging.info("fromfile - %s from %s" % (self.origname, whichmodule(2)))
        self.fromfile(self.cfile)
        self.init()
        if self.owner: logging.info("owner is %s" % self.owner)
        if "uuid" not in self: self.setuuid()
        if "cfile" not in self: self.cfile = self.setcfile(self.origdir, self.origname) 
        assert self.cfile

    def setcfile(self, ddir, filename):
        self.filename = filename or 'mainconfig'
        self.datadir = ddir or getdatadir()
        self.dir = self.datadir + os.sep + 'config'
        self.cfile = self.dir + os.sep + filename

    def setuuid(self, save=True):
        logging.debug("setting uuid")
        self.uuid = str(uuid.uuid4())
        if save: self.save()

    def __deepcopy__(self, a):
        """ accessor function. """
        cfg = Config(self.filename, input=self, nolog=True)
        return cfg

    def __getitem__(self, item):
        """ accessor function. """
        if item not in self: return None
        else: return LazyDict.__getitem__(self, item)

    def merge(self, cfg):
        """ merge in another cfg. """
        self.update(cfg)

    def set(self, item, value):
        """ set item to value. """
        LazyDict.__setitem__(self, item, value)

    def fromdb(self):
        """ read config from database. """
        from ggz.lib.persist import Persist
        tmp = Persist(self.cfile)
        logging.debug("fromdb - %s - %s" % (self.cfile, tmp.data.tojson()))
        self.update(tmp.data)

    def todb(self):
        """ save config to database. """
        cp = dict(self)
        del cp['jsondb']
        if not self.jsondb:
            from ggz.lib.persist import Persist
            self.jsondb = Persist(self.cfile)
        self.jsondb.data = cp
        self.jsondb.save()

    def fromfile(self, filename=None):
        """ read config object from filename. """
        curline = ""
        fname = filename or self.cfile
        if not fname: raise Exception(" %s - %s" % (self.cfile, self.dump()))
        if not os.path.exists(fname): logging.warn("config file %s doesn't exist yet" % fname) ; return False 
        comment = ""
        for line in open(fname, 'r'):
            curline = line
            curline = curline.strip()
            if curline == "": continue
            if curline.startswith('#'): comment = curline; continue
            if True:
                try:
                    key, value = curline.split('=', 1)
                    kkey = key.strip()
                    self[kkey] = json.loads(str(value.strip()))
                    if comment: self._comments[kkey] = comment 
                    comment = ""
                except ValueError: logging.error("skipping line - unable to parse: %s" % line)
        #self.cfile = fname
        return

    def tofile(self, filename=None, stdout=False):
        """ save config object to file. """
        if not filename: filename = self.cfile
        if not filename: raise Exception("no cfile found  - %s" % whichmodule(3))
        logging.warn("saving %s" % filename)
        if filename.startswith(os.sep): d = [os.sep,]
        else: d = []
        for p in filename.split(os.sep)[:-1]:
            if not p: continue
            d.append(p)
            ddir = os.sep.join(d)
            if not os.path.isdir(ddir):
                logging.debug("persist - creating %s dir" % ddir)
                try: os.mkdir(ddir)
                except OSError as ex:
                    logging.error("persist - not saving - failed to make %s - %s" % (ddir, str(ex)))
                    return
        written = []
        curitem = None
        later = []
        try:
            if stdout: configtmp = sys.stdout
            else: configtmp = open(filename + '.tmp', 'w')
            configtmp.write('# ===========================================================\n#\n')
            configtmp.write("# GGZBOT CONFIGURATION FILE - %s\n" % filename)
            configtmp.write("#\n")
            configtmp.write('# last changed on %s\n#\n' % time.ctime(time.time()))
            configtmp.write("# This file contains configration data for the GGZBOT.\n")
            configtmp.write('# Variables are defined by "name = json value" pairs.\n')
            configtmp.write('# Make sure to use " in strings.\n#\n')
            configtmp.write('# The bot can edit this file!.\n#\n')
            configtmp.write('# ===========================================================\n\n')
            teller = 0
            keywords = list(self.keys())
            keywords.sort()
            for keyword in keywords:
                value = self[keyword]
                if keyword in written: continue
                if keyword in ['origdir', 'origname', 'issaved', 'blacklist', 'whitelist', 'followlist', 'uuid', 'whitelist', 'datadir', 'name', 'createdfrom', 'cfile', 'filename', 'dir', 'isdb']: later.append(keyword) ; continue
                if keyword == 'jsondb': continue
                if keyword == 'optionslist': continue
                if keyword == 'gatekeeper': continue
                if keyword == "_comments": continue
                if self._comments and keyword in self._comments:
                    configtmp.write(self._comments[keyword] + "\n")
                curitem = keyword
                try: configtmp.write('%s = %s\n' % (keyword, json.dumps(value)))
                except TypeError: logging.error("%s - can't serialize %s" % (filename, keyword)) ; continue
                teller += 1
                #configtmp.write("\n")
            configtmp.write('\n\n# ============================================================\n#\n')
            configtmp.write("# bot generated stuff.\n#\n")
            configtmp.write('# ============================================================\n\n')
            for keyword in later:
                if self._comments and keyword in self._comments:
                    configtmp.write(self._comments[keyword] + "\n")
                curitem = keyword
                value = self[keyword]
                try: configtmp.write(keyword + " = " + json.dumps(value) + "\n")
                except TypeError: logging.error("%s - can't serialize %s" % (filename, keyword)) ; continue
                teller += 1
                #configtmp.write("\n")
            if not "mainconfig" in filename and self._comments:
                try:
                    configtmp.write('\n\n# ============================================================\n#\n')
                    configtmp.write("# possible other config variables.\n#\n")
                    configtmp.write('# ============================================================\n\n')
                    items = list(self._comments.keys())
                    keys = list(self.keys())
                    do = []
                    for var in items:
                        if var not in keys: do.append(var)
                    do.sort()
                    for var in do:
                         configtmp.write("# %s -=- %s\n" % (var, self._comments[var]))
                    configtmp.write("\n\n")
                except Exception as ex: handle_exception()
            else: configtmp.write("\n\n# ggzbot can run multiple bots at once. see %s/config/fleet for their configurations.\n\n" % self.origdir)
            if not stdout: 
                configtmp.close()
                os.rename(filename + '.tmp', filename)
            return teller

        except Exception as ex:
            handle_exception()
            logging.error("ERROR WRITING %s CONFIG FILE: %s .. %s" % (self.cfile, str(ex), curitem))

    @savelocked
    def save(self):
        """ save the config. """
        logging.info("save called from %s" % calledfrom(sys._getframe(2)))
        self.issaved = True
        if self.isdb: self.todb()
        else: self.tofile(self.cfile)
     
    def load_config(self, verbose=False):
        """ load the config file. """
        if self.isdb: self.fromdb()
        else: self.fromfile(self.filename)
        self.init()
        if verbose: logging.debug('%s' % self.dump())

    def init(self):
        """ initialize the config object. """
        if not self._comments: self._comments = {}
        if self.filename == 'mainconfig':
            self._comments["whitelist"] = "# - whitelist used to allow ips .. bot maintains this"
            self.setdefault("whitelist", [])
            self._comments["blacklist"] = "# - blacklist used to deny ips .. bot maintains this"
            self.setdefault("blacklist", [])
            self.setdefault('owner', [])
            self._comments["loglist"] = "# - loglist .. maintained by the bot."
            self.setdefault('loglist',  [])
            self._comments["loglevel"] = "# - loglevel of all bots"
            self.setdefault('loglevel',  "warn")
            self._comments["loadlist"] = "# - loadlist .. not used yet."
            self.setdefault('loadlist', [])
            self._comments["quitmsg"] = "# - message to send on quit"
            self.setdefault('quitmsg', "http://ggzbot.googlecode.com")
            self._comments["dotchars"] = "# - characters to used as seperator."
            self.setdefault('dotchars',  ", ")
            self._comments["floodallow"] = "# - whether the bot is allowed to flood."
            self.setdefault('floodallow', 1)
            self._comments["auto_register"] = "# - enable automatic registration of new users."
            self.setdefault('auto_register', 0)
            self._comments["guestasuser"] = "# - enable this to give new users the USER permission besides GUEST."
            self.setdefault('guestasuser', 0)
            self._comments["globalcc"] = "# - global control character"
            self.setdefault('globalcc', "")
            self._comments["app_id"] = "# - application id used by appengine."
            self.setdefault('app_id', "ggzbot")
            self._comments["appname"] = "# - application name as used by the bot."
            self.setdefault('appname', "GGZBOT")
            self._comments["domain"] = "# - domain .. used for WAVE."
            self.setdefault('domain', "")
            self._comments["color"] = "# - color used in the webconsole."
            self.setdefault('color', "")
            self._comments["colors"] = "# - enable colors in logging."
            self.setdefault('colors', "")
            self._comments["memcached"] = "# - enable memcached."
            self.setdefault('memcached', 0)
            self._comments["allowrc"] = "# - allow execution of rc files."
            self.setdefault('allowrc', 0)
            self._comments["allowremoterc"] = "# - allow execution of remote rc files."
            self.setdefault('allowremoterc', 0)
            self._comments['dbenable'] = "# - enable database support"
            self.setdefault('dbenable', 1)
            self._comments['dbtype'] = "# - type of database .. sqlite or mysql at this time."
            self.setdefault('dbtype', 'sqlite')
            self._comments['dbname'] = "# - database name"
            self.setdefault('dbname', "main.db")
            self._comments['dbhost'] = "# - database hostname"
            self.setdefault('dbhost', "localhost") 
            self._comments['dbuser'] = "# - database user"
            self.setdefault('dbuser', "bart")
            self._comments['dbpasswd'] = "# - database password"
            self.setdefault('dbpasswd', "mekker2")
            self._comments['ticksleep'] = "# - nr of seconds to sleep before creating a TICK event."
            self.setdefault('ticksleep', 1)
            self._comments['bindhost'] = "# - host to bind to"
            self.setdefault("bindhost", "")
        self['createdfrom'] = whichmodule()
        if 'xmpp' in self.cfile: self.setdefault('fulljids', 1)
        if 'fleet' in self.cfile:
            self.setdefault('disable', 1)
            self.setdefault("owner", [])
            self.setdefault("user", "")
            self.setdefault("host", "")
            self.setdefault("server", "")
            self.setdefault("ssl", 0)
            self.setdefault("ipv6", 0)
            self.setdefault("channels", [])
        self.setdefault("port", "")
        self.setdefault("password", "")
        self._comments['datadir'] = "# - directory to store bot data in."
        self._comments["owner"] = "# - owner of the bot."
        self._comments["uuid"] = "# - bot generated uuid for this config file."
        self._comments["user"] = "# - user used to login on xmpp networks."
        self._comments["host"] = "# - host part of the user, derived from user var."
        self._comments["server"] = "# - server to connect to (only when different from users host)."
        self._comments["password"] = "# - password to use in authing the bot."
        self._comments["port"] = "# - port to connect to (IRC)."
        self._comments["ssl"] = "# - whether to enable ssl (set to 1 to enable)."
        self._comments["ipv6"] = "# - whether to enable ssl (set to 1 to enable)."
        self._comments["name"] = "# - the name of the bot."
        self._comments["disable"] = "# - set this to 0 to enable the bot."
        self._comments["followlist"] = "# - who to follow on the bot .. bot maintains this list."
        self._comments["networkname"] = "# - networkname .. not used right now."
        self._comments["type"] = "# - the bot's type."
        self._comments["nick"] = "# - the bot's nick."
        self._comments["channels"] = "# - channels to join."
        self._comments["cfile"] = "# - filename of this config file. edit this when you move this file."
        self._comments["createdfrom"] = "# - function that created this config file. bot generated"
        self._comments["dir"] = "# - directory in which this config file lives."
        self._comments["isdb"] = "# - whether this config file lives in the database and not on file."
        self._comments["filename"] = "# - filename of this config file."
        self._comments["username"] = "# - username of the bot."
        self._comments["fulljids"] = "# - use fulljids of bot users (used in non anonymous conferences."
        self._comments["servermodes"] = "# - string of modes to send to the server after connect."
        self._comments["realname"] = "# - name used in the ident of the bot."
        self._comments["onconnect"] = "# - string to send to server after connect."
        self._comments["onconnectmode"] = "# - MODE string to send to server after connect."
        self._comments["realname"] = "# - mode string to send to the server after connect."
        self._comments["issaved"] = "# - whether this config file has been saved. "
        self._comments["origdir"] = "# - original datadir for this configfile. "
        self._comments["origname"] = "# - displayable name of the config file name. "
        return self
        
    def reload(self):
        """ reload the config file. """
        self.load_config()
        return self

def ownercheck(userhost):
    """ check whether userhost is a owner. """
    if not userhost: return False
    if userhost in cfg['owner']: return True
    return False

mainconfig = None

def getmainconfig(ddir=None):
    global mainconfig
    if not mainconfig: mainconfig = Config("mainconfig", ddir=ddir)
    if "issaved" not in mainconfig: mainconfig.save()
    return mainconfig

irctemplate = """# =====================================================
#
# GGZBOT CONFIGURATION FILE - 
#
# last changed on 
#
# This file contains configration data for the GGZBOT.
# Variables are defined by "name = json value" pairs.
# Make sure to use " in strings.
# The bot can edit this file!
#
# =====================================================


# - to enable put this to 0
disable = 1

# - the bot's nick.
nick = "ggzbot"

# - owner of the bot.
owner = []

# - port to connect to (IRC).
port = 6667

# - server to connect to (on jabber only when different that host.
server = "localhost"

# - the bot's type.
type = "irc"

# - username of the bot.
username = "ggzbot"

# - ssl enabled or not
ssl = 0

# - ipv6 enabled or not 
ipv6 = 0

# - name use in ident of the bot
realname = "ggzbot"

# - string of modes send to the server on connect
servermodes = ""

# =====================================================
#
# bot generated stuff.
#
# =====================================================

"""

xmpptemplate = """# =====================================================
#
# GGZBOT CONFIGURATION FILE - 
#
# last changed on 
#
# This file contains configration data for the GGZBOT.
# Variables are defined by "name = json value" pairs.
# Make sure to use " in strings.
# The bot can edit this file!
#
# =====================================================

# - channels to join
channels = []

# - to enable put this to 0
disable = 1

# - the bot's nick.
nick = "ggzbot"

# - owner of the bot.
owner = []

# - use fulljids of bot users (used in non anonymous conferences.
fulljids = 1

# password used to auth on the server.
password = ""

# - server to connect to (on jabber only when different that users host.
server = ""

# - the bot's type.
type = "sxmpp"

# - user used to login on xmpp networks.
user = ""

# =====================================================
#
# bot generated stuff.
#
# =====================================================

"""

sleektemplate = """# =====================================================
#
# GGZBOT CONFIGURATION FILE - 
#
# last changed on 
#
# This file contains configration data for the GGZBOT.
# Variables are defined by "name = json value" pairs.
# Make sure to use " in strings.
# The bot can edit this file!
#
# =====================================================

# - channels to join
channels = []

# - to enable put this to 0
disable = 1

# - the bot's nick.
nick = "ggzbot"

# - owner of the bot.
owner = []

# - use fulljids of bot users (used in non anonymous conferences.
fulljids = 1

# password used to auth on the server.
password = ""

# - server to connect to (on jabber only when different that users host.
server = ""

# - the bot's type.
type = "sleek"

# - user used to login on xmpp networks.
user = ""

# =====================================================
#
# bot generated stuff.
#
# =====================================================

"""

def makedefaultconfig(type, ddir=None):
    filename = 'config'
    datadir = ddir or getdatadir()
    dir = datadir + os.sep + 'config'
    ttype = "default-%s" % type
    cfile = dir + os.sep + "fleet" + os.sep + ttype + os.sep + filename
    logging.warn("creating default config for type %s in %s" % (type, cfile))
    splitted = cfile.split(os.sep)
    mdir = "" 
    for i in splitted[:-1]:
        mdir += "%s%s" % (i, os.sep)
        if not os.path.isdir(mdir): os.mkdir(mdir)
    logging.debug("filename is %s" % cfile)
    f = open(cfile, "w")
    if type == "irc": f.write(irctemplate) ; f.close()
    elif type == "sxmpp": f.write(xmpptemplate) ; f.close()
    elif type == "sleek": f.write(sleektemplate) ; f.close()
    else: raise Exception("no such bot type: %s" % type)



# jsb/lib/nextid.py
#
#

""" provide increasing counters """

## basic imports

import os

## Nextid class

class Nextid(object):

    """ counters by name """

    def __init__(self, fname):
        self.data = {}

    def get(self, item):
        """ get counter for item """
        item = item.lower()
        try:
            result = self.data[item]
        except KeyError:
            return None 
        return result

    def set(self, item, number):
        """ set counter of item to number """
        item = item.lower()
        try:
            self.data[item] = int(number)
        except ValueError:
            return 0
        self.save()
        return 1

    def next(self, item):
        """ get increment of item counter """
        item = item.lower()
        try:
            self.data[item] += 1
        except KeyError:
            self.data[item] = 1
        self.save()
        return self.data[item]

    def nextlist(self, item, nr):
        """ get increment of item counter """
        item = item.lower()
        try:
            start = self.data[item] + 1
        except KeyError:
            start = 1
        stop = start + nr
        l = list(range(start, stop))
        self.data[item] = stop - 1
        self.save()
        return l

    def save(self):
        pass

nextid = Nextid('notused')



# ggz/utils/source.py
#
#

""" get the location of a source """

## ggz imports

from ggz.utils.exception import handle_exception

## basic imports

import os
import os.path
import logging
import sys

## getsource function

def getsource(mod):
    if not os.getcwd() in sys.path: sys.path.insert(0, os.getcwd())
    source = None
    splitted = mod.split(".")
    if len(splitted) == 1: splitted.append("")
    thedir = os.path.abspath(mod.replace(".", os.sep))
    if os.path.isdir(thedir): source = thedir
    if source and os.path.exists(source):
        logging.info("datadir - source is %s" % source)
        return source
    if not source:
        try:
            import pkg_resources
            source = pkg_resources.resource_filename(".".join(splitted[:len(splitted)-1]), splitted[-1])
        except ImportError:
            try:
                import jsb.contrib.pkg_resources as pkg_resources
                source = pkg_resources.resource_filename(".".join(splitted[:len(splitted)-1]), splitted[-1])
            except ImportError: pass
    logging.info("datadir - source is %s" % source)
    return source



# -*- coding: utf-8 -*-
"""
    sleekxmpp.clientxmpp
    ~~~~~~~~~~~~~~~~~~~~

    This module provides XMPP functionality that
    is specific to client connections.

    Part of SleekXMPP: The Sleek XMPP Library

    :copyright: (c) 2011 Nathanael C. Fritz
    :license: MIT, see LICENSE for more details
"""

from __future__ import absolute_import, unicode_literals

import logging
import base64
import sys
import hashlib
import random
import threading

import sleekxmpp
from sleekxmpp import plugins
from sleekxmpp import stanza
from sleekxmpp import features
from sleekxmpp.basexmpp import BaseXMPP
from sleekxmpp.stanza import *
from sleekxmpp.xmlstream import XMLStream, RestartStream
from sleekxmpp.xmlstream import StanzaBase, ET, register_stanza_plugin
from sleekxmpp.xmlstream.matcher import *
from sleekxmpp.xmlstream.handler import *

# Flag indicating if DNS SRV records are available for use.
try:
    import dns.resolver
except ImportError:
    DNSPYTHON = False
else:
    DNSPYTHON = True


log = logging.getLogger(__name__)


class ClientXMPP(BaseXMPP):

    """
    SleekXMPP's client class. (Use only for good, not for evil.)

    Typical use pattern:

    .. code-block:: python

        xmpp = ClientXMPP('user@server.tld/resource', 'password')
        # ... Register plugins and event handlers ...
        xmpp.connect()
        xmpp.process(block=False) # block=True will block the current
                                  # thread. By default, block=False

    :param jid: The JID of the XMPP user account.
    :param password: The password for the XMPP user account.
    :param ssl: **Deprecated.**
    :param plugin_config: A dictionary of plugin configurations.
    :param plugin_whitelist: A list of approved plugins that 
                    will be loaded when calling 
                    :meth:`~sleekxmpp.basexmpp.BaseXMPP.register_plugins()`.
    :param escape_quotes: **Deprecated.**
    """

    def __init__(self, jid, password, ssl=False, plugin_config={},
                 plugin_whitelist=[], escape_quotes=True, sasl_mech=None):
        BaseXMPP.__init__(self, jid, 'jabber:client')

        self.set_jid(jid)
        self.password = password
        self.escape_quotes = escape_quotes
        self.plugin_config = plugin_config
        self.plugin_whitelist = plugin_whitelist
        self.default_port = 5222

        self.stream_header = "<stream:stream to='%s' %s %s version='1.0'>" % (
                self.boundjid.host,
                "xmlns:stream='%s'" % self.stream_ns,
                "xmlns='%s'" % self.default_ns)
        self.stream_footer = "</stream:stream>"

        self.features = set()
        self._stream_feature_handlers = {}
        self._stream_feature_order = []

        #TODO: Use stream state here
        self.authenticated = False
        self.sessionstarted = False
        self.bound = False
        self.bindfail = False

        self.add_event_handler('connected', self._handle_connected)

        self.register_stanza(StreamFeatures)

        self.register_handler(
                Callback('Stream Features',
                         MatchXPath('{%s}features' % self.stream_ns),
                         self._handle_stream_features))
        self.register_handler(
                Callback('Roster Update',
                         MatchXPath('{%s}iq/{%s}query' % (
                             self.default_ns,
                             'jabber:iq:roster')),
                         self._handle_roster))

        # Setup default stream features
        self.register_plugin('feature_starttls')
        self.register_plugin('feature_bind')
        self.register_plugin('feature_session')
        self.register_plugin('feature_mechanisms',
                pconfig={'use_mech': sasl_mech} if sasl_mech else None)

    def connect(self, address=tuple(), reattempt=True,
                use_tls=True, use_ssl=False):
        """Connect to the XMPP server.

        When no address is given, a SRV lookup for the server will
        be attempted. If that fails, the server user in the JID
        will be used.

        :param address   -- A tuple containing the server's host and port.
        :param reattempt: If ``True``, repeat attempting to connect if an
                         error occurs. Defaults to ``True``.
        :param use_tls: Indicates if TLS should be used for the
                        connection. Defaults to ``True``.
        :param use_ssl: Indicates if the older SSL connection method
                        should be used. Defaults to ``False``.
        """
        self.session_started_event.clear()
        if not address:
            address = (self.boundjid.host, 5222)

        return XMLStream.connect(self, address[0], address[1],
                                 use_tls=use_tls, use_ssl=use_ssl,
                                 reattempt=reattempt)

    def get_dns_records(self, domain, port=None):
        """Get the DNS records for a domain, including SRV records.

        :param domain: The domain in question.
        :param port: If the results don't include a port, use this one.
        """
        if port is None:
            port = self.default_port
        if DNSPYTHON:
            try:
                record = "_xmpp-client._tcp.%s" % domain
                answers = []
                for answer in dns.resolver.query(record, dns.rdatatype.SRV):
                    address = (answer.target.to_text()[:-1], answer.port)
                    answers.append((address, answer.priority, answer.weight))
            except (dns.resolver.NXDOMAIN, dns.resolver.NoAnswer):
                log.warning("No SRV records for %s", domain)
                answers = super(ClientXMPP, self).get_dns_records(domain, port)
            except dns.exception.Timeout:
                log.warning("DNS resolution timed out " + \
                            "for SRV record of %s", domain)
                answers = super(ClientXMPP, self).get_dns_records(domain, port)
            return answers
        else:
            log.warning("dnspython is not installed -- " + \
                        "relying on OS A record resolution")
            return [((domain, port), 0, 0)]

    def register_feature(self, name, handler, restart=False, order=5000):
        """Register a stream feature handler.

        :param name: The name of the stream feature.
        :param handler: The function to execute if the feature is received.
        :param restart: Indicates if feature processing should halt with
                        this feature. Defaults to ``False``.
        :param order: The relative ordering in which the feature should
                      be negotiated. Lower values will be attempted
                      earlier when available.
        """
        self._stream_feature_handlers[name] = (handler, restart)
        self._stream_feature_order.append((order, name))
        self._stream_feature_order.sort()

    def update_roster(self, jid, name=None, subscription=None, groups=[],
                            block=True, timeout=None, callback=None):
        """Add or change a roster item.

        :param jid: The JID of the entry to modify.
        :param name: The user's nickname for this JID.
        :param subscription: The subscription status. May be one of
                             ``'to'``, ``'from'``, ``'both'``, or
                             ``'none'``. If set to ``'remove'``,
                             the entry will be deleted.
        :param groups: The roster groups that contain this item.
        :param block: Specify if the roster request will block
                      until a response is received, or a timeout
                      occurs. Defaults to ``True``.
        :param timeout: The length of time (in seconds) to wait
                        for a response before continuing if blocking
                        is used. Defaults to 
            :attr:`~sleekxmpp.xmlstream.xmlstream.XMLStream.response_timeout`.
        :param callback: Optional reference to a stream handler function.
                         Will be executed when the roster is received.
                         Implies ``block=False``.
        """
        return self.client_roster.update(jid, name, subscription, groups,
                                         block, timeout, callback)

    def del_roster_item(self, jid):
        """Remove an item from the roster.
        
        This is done by setting its subscription status to ``'remove'``.

        :param jid: The JID of the item to remove.
        """
        return self.client_roster.remove(jid)

    def get_roster(self, block=True, timeout=None, callback=None):
        """Request the roster from the server.

        :param block: Specify if the roster request will block until a
                      response is received, or a timeout occurs.
                      Defaults to ``True``.
        :param timeout: The length of time (in seconds) to wait for a response
                        before continuing if blocking is used.
                        Defaults to 
            :attr:`~sleekxmpp.xmlstream.xmlstream.XMLStream.response_timeout`.
        :param callback: Optional reference to a stream handler function. Will
                         be executed when the roster is received.
                         Implies ``block=False``.
        """
        iq = self.Iq()
        iq['type'] = 'get'
        iq.enable('roster')
        response = iq.send(block, timeout, callback)

        if callback is None:
            return self._handle_roster(response, request=True)

    def _handle_connected(self, event=None):
        #TODO: Use stream state here
        self.authenticated = False
        self.sessionstarted = False
        self.bound = False
        self.bindfail = False
        self.features = set()

    def _handle_stream_features(self, features):
        """Process the received stream features.

        :param features: The features stanza.
        """
        for order, name in self._stream_feature_order:
            if name in features['features']:
                handler, restart = self._stream_feature_handlers[name]
                if handler(features) and restart:
                    # Don't continue if the feature requires
                    # restarting the XML stream.
                    return True

    def _handle_roster(self, iq, request=False):
        """Update the roster after receiving a roster stanza.

        :param iq: The roster stanza.
        :param request: Indicates if this stanza is a response
                        to a request for the roster, and not an
                        empty acknowledgement from the server.
        """
        if iq['type'] == 'set' or (iq['type'] == 'result' and request):
            for jid in iq['roster']['items']:
                item = iq['roster']['items'][jid]
                roster = self.roster[iq['to'].bare]
                roster[jid]['name'] = item['name']
                roster[jid]['groups'] = item['groups']
                roster[jid]['from'] = item['subscription'] in ['from', 'both']
                roster[jid]['to'] = item['subscription'] in ['to', 'both']
                roster[jid]['pending_out'] = (item['ask'] == 'subscribe')
            self.event('roster_received', iq)

        self.event("roster_update", iq)
        if iq['type'] == 'set':
            iq.reply()
            iq.enable('roster')
            iq.send()
        return True


# To comply with PEP8, method names now use underscores.
# Deprecated method names are re-mapped for backwards compatibility.
ClientXMPP.updateRoster = ClientXMPP.update_roster
ClientXMPP.delRosterItem = ClientXMPP.del_roster_item
ClientXMPP.getRoster = ClientXMPP.get_roster
ClientXMPP.registerFeature = ClientXMPP.register_feature



# jsb/plugs/common/search.py
#
#

""" access stats data from the spider plugin. """

## jsb imports

from ggz.lib.commands import cmnds
from ggz.lib.examples import examples
from ggz.lib.persist import PersistCollection, Persist
from ggz.lib.datadir import getdatadir
from ggz.utils.name import stripname
from ggz.utils.exception import handle_exception
from ggz.utils.statdict import StatDict
from ggz.utils.urldata import UrlData

## basic imports

import os

## searchobj function

def makestats(objs, target, skip=[]):
    stats = StatDict()
    if not objs: return stats
    ignore = []
    res = []
    count = {}
    for t in target:
        for u in objs:
            if u in ignore: continue
            if u and u.data and u.data.txt:
                cont = False
                for s in skip:
                    if s in u.data.url: cont = True ; break
                if cont: continue
                if t in u.data.txt:
                    res.append(u)
                else:
                    ignore.append(u)
                    try: res.remove(u)
                    except ValueError: pass 
    for item in res:
        c = 0
        if not item in ignore:
            for t in target: c += item.data.txt.count(t)
            stats.upitem(item.data.url, c)
    return stats

def stats_response(stats, target=[], skip=[]):
    r = []
    first = []
    todo = stats.top(start=3)
    if not todo: todo = stats.top()
    for res in todo:
        cont = False
        for s in skip:
            if s in str(res): cont = True
        if cont: continue
        url = res[0]
        count = res[1]
        if not url.endswith(".html"): continue
        if "genindex" in str(url): continue
        splitted = url.split("/")
        if target and target[-1] in splitted[-1]: first.append("%s (%s)" % (url, count))
        else: r.append("%s (%s)" % (url, count))
    first.extend(r)
    return first

## search command

def handle_search(bot, event):
    if not event.options: event.makeoptions()
    all = event.options.all
    res = []
    target = event.args
    if not target: event.missing("<search words seperated by space>") ; return
    coll = PersistCollection(getdatadir() + os.sep + 'spider' + os.sep + "data")
    files = coll.filenames(target)
    if files:
        for f in files:
            try: res.append(Persist(f).data.url)
            except AttributeError as ex: continue
    objs = coll.search('txt', event.rest)
    if not objs: objs = list(coll.objects().values())
    stats = makestats(objs, target, res)
    urls = stats_response(stats, target)
    res.extend(urls)
    if res:
        if len(res) < 4 or all: event.reply("found %s urls: " % len(res), res, dot=" -or- ")
        else: event.reply("found %s urls, use --all for more: " % len(res), res[:3], dot=" -or- ")
    else: event.reply("no urls found")

cmnds.add("search", handle_search, ["OPER", "USER", "GUEST"])
examples.add("search", "search scanned sites by the spider", "search license")



# gozerbot/ircevent.py
#
#
# http://www.irchelp.org/irchelp/rfc/rfc2812.txt

""" an ircevent is extracted from the IRC string received from the server. """

## jsb imports

from ggz.utils.generic import toenc, fromenc, strippedtxt, fix_format
from ggz.lib.eventbase import EventBase

## basic imports

import time
import re
import types
import copy
import logging

## defines

cpy = copy.deepcopy

## Ircevent class

class IrcEvent(EventBase):

    """ represents an IRC event. """

    def __deepcopy__(self, bla):
        e = IrcEvent()
        e.copyin(self)
        return e

    def parse(self, bot, rawstr):
        """ parse raw string into ircevent. """
        self.bottype = "irc"
        self.bot = bot
        self.ttl = 2
        self.speed = 4
        rawstr = rawstr.rstrip()
        splitted = re.split('\s+', rawstr)
        if not rawstr[0] == ':':
            assert bot.cfg
            splitted.insert(0, ":%s!%s@%s" % (bot.cfg.nick, bot.cfg.username, bot.cfg.server))
            rawstr = ":%s!%s@%s %s" % (bot.cfg.nick, bot.cfg.username, bot.cfg.server, rawstr)
        self.prefix = splitted[0][1:]
        nickuser = self.prefix.split('!')
        try: 
            self.userhost = nickuser[1]
            self.nick = nickuser[0]
        except IndexError: self.userhost = None ; self.nick = None ; self.isservermsg = True
        self.cmnd = splitted[1]
        self.cbtype = self.cmnd
        if self.cmnd in pfc:
            self.arguments = splitted[2:pfc[self.cmnd]+2]
            txtsplit = re.split('\s+', rawstr, pfc[self.cmnd]+2)
            self.txt = txtsplit[-1]
        else:
            self.arguments = splitted[2:]
        if self.arguments: self.target = self.arguments[0]
        self.postfix = ' '.join(self.arguments)
        if self.target and self.target.startswith(':'): self.txt = ' '.join(self.arguments)
        if self.txt:
            if self.txt[0] == ":": self.txt = self.txt[1:]
            if self.txt: self.usercmnd = self.txt.split()[0]
        if self.cmnd == 'PING': self.speed = 9
        if self.cmnd == 'PRIVMSG':
            self.channel = self.arguments[0]
            if '\001' in self.txt: self.isctcp = True
        elif self.cmnd == 'JOIN' or self.cmnd == 'PART':
            if self.arguments: self.channel = self.arguments[0]
            else: self.channel = self.txt
        elif self.cmnd == 'MODE': self.channel = self.arguments[0]
        elif self.cmnd == 'TOPIC': self.channel = self.arguments[0]
        elif self.cmnd == 'KICK': self.channel = self.arguments[0]
        elif self.cmnd == '353': self.channel = self.arguments[2]
        elif self.cmnd == '324': self.channel = self.arguments[1]
        if self.userhost:
            self.ruserhost = self.userhost
            self.stripped = self.userhost
            self.auth = self.userhost
            try: self.hostname = self.userhost.split("@")[1]
            except: self.hostname = None
        self.origtxt = self.txt
        if self.channel:
            self.channel = self.channel.strip()
            self.origchannel = self.channel
            if self.channel == self.bot.cfg.nick:
                logging.warn("irc - msg detected - setting channel to %s" % self.userhost)
                self.msg = True
                self.channel = self.userhost
        if not self.channel:
            for c in self.arguments:
                if c.startswith("#"): self.channel = c
        try:
            nr = int(self.cmnd)
            if nr > 399 and not nr == 422: logging.error('%s - %s - %s - %s' % (self.bot.cfg.name, self.cmnd, self.arguments, self.txt))
        except ValueError: pass
        self.prepare()
        return self

    def reply(self, txt, result=[], event=None, origin="", dot=", ", nr=375, extend=0, *args, **kwargs):
        """ reply to this event """
        if self.isdcc: self.bot.say(self.sock, txt, result, 'msg', self, nr, extend, dot, *args, **kwargs)
        elif self.msg: self.bot.say(self.nick, txt, result, 'msg', self, nr, extend, dot, *args, **kwargs)
        elif self.silent or (self.chan and self.chan.data and self.chan.data.silent): self.bot.say(self.nick, txt, result, 'msg', self, nr, extend, dot, *args, **kwargs)
        else: self.bot.say(self.channel, txt, result, 'msg', self, nr, extend, dot, *args, **kwargs)
        return self

## postfix count - how many arguments

pfc = {}
pfc['NICK'] = 0
pfc['QUIT'] = 0
pfc['SQUIT'] = 1
pfc['JOIN'] = 0
pfc['PART'] = 1
pfc['TOPIC'] = 1
pfc['KICK'] = 2
pfc['PRIVMSG'] = 1
pfc['NOTICE'] = 1
pfc['SQUERY'] = 1
pfc['PING'] = 0
pfc['ERROR'] = 0
pfc['AWAY'] = 0
pfc['WALLOPS'] = 0
pfc['INVITE'] = 1
pfc['001'] = 1
pfc['002'] = 1
pfc['003'] = 1
pfc['004'] = 4
pfc['005'] = 15
pfc['302'] = 1
pfc['303'] = 1
pfc['301'] = 2
pfc['305'] = 1
pfc['306'] = 1
pfc['311'] = 5
pfc['312'] = 3
pfc['313'] = 2
pfc['317'] = 3
pfc['318'] = 2
pfc['319'] = 2
pfc['314'] = 5
pfc['369'] = 2
pfc['322'] = 3
pfc['323'] = 1
pfc['325'] = 3
pfc['324'] = 4
pfc['331'] = 2
pfc['332'] = 2
pfc['341'] = 3
pfc['342'] = 2
pfc['346'] = 3
pfc['347'] = 2
pfc['348'] = 3
pfc['349'] = 2
pfc['351'] = 3
pfc['352'] = 7
pfc['315'] = 2
pfc['353'] = 3
pfc['366'] = 2
pfc['364'] = 3
pfc['365'] = 2
pfc['367'] = 2
pfc['368'] = 2
pfc['371'] = 1
pfc['374'] = 1
pfc['375'] = 1
pfc['372'] = 1
pfc['376'] = 1
pfc['381'] = 1
pfc['382'] = 2
pfc['383'] = 5
pfc['391'] = 2
pfc['392'] = 1
pfc['393'] = 1
pfc['394'] = 1
pfc['395'] = 1
pfc['262'] = 3
pfc['242'] = 1
pfc['235'] = 3
pfc['250'] = 1
pfc['251'] = 1
pfc['252'] = 2
pfc['253'] = 2
pfc['254'] = 2
pfc['255'] = 1
pfc['256'] = 2
pfc['257'] = 1
pfc['258'] = 1
pfc['259'] = 1
pfc['263'] = 2
pfc['265'] = 1
pfc['266'] = 1
pfc['401'] = 2
pfc['402'] = 2
pfc['403'] = 2
pfc['404'] = 2
pfc['405'] = 2
pfc['406'] = 2
pfc['407'] = 2
pfc['408'] = 2
pfc['409'] = 1
pfc['411'] = 1
pfc['412'] = 1
pfc['413'] = 2
pfc['414'] = 2
pfc['415'] = 2
pfc['421'] = 2
pfc['422'] = 1
pfc['423'] = 2
pfc['424'] = 1
pfc['431'] = 1
pfc['432'] = 2
pfc['433'] = 2
pfc['436'] = 2
pfc['437'] = 2
pfc['441'] = 3
pfc['442'] = 2
pfc['443'] = 3
pfc['444'] = 2
pfc['445'] = 1
pfc['446'] = 1
pfc['451'] = 1
pfc['461'] = 2
pfc['462'] = 1
pfc['463'] = 1
pfc['464'] = 1
pfc['465'] = 1
pfc['467'] = 2
pfc['471'] = 2
pfc['472'] = 2
pfc['473'] = 2
pfc['474'] = 2
pfc['475'] = 2
pfc['476'] = 2
pfc['477'] = 2
pfc['478'] = 3
pfc['481'] = 1
pfc['482'] = 2
pfc['483'] = 1
pfc['484'] = 1
pfc['485'] = 1
pfc['491'] = 1
pfc['501'] = 1
pfc['502'] = 1
pfc['700'] = 2



# -*- coding: utf-8 -*-
"""
    sleekxmpp.xmlstream.handler.waiter
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Part of SleekXMPP: The Sleek XMPP Library

    :copyright: (c) 2011 Nathanael C. Fritz
    :license: MIT, see LICENSE for more details
"""

import logging
try:
    import queue
except ImportError:
    import Queue as queue

from sleekxmpp.xmlstream import StanzaBase
from sleekxmpp.xmlstream.handler.base import BaseHandler


log = logging.getLogger(__name__)


class Waiter(BaseHandler):

    """
    The Waiter handler allows an event handler to block until a
    particular stanza has been received. The handler will either be
    given the matched stanza, or ``False`` if the waiter has timed out.

    :param string name: The name of the handler.
    :param matcher: A :class:`~sleekxmpp.xmlstream.matcher.base.MatcherBase`
                    derived object for matching stanza objects.
    :param stream: The :class:`~sleekxmpp.xmlstream.xmlstream.XMLStream`
                   instance this handler should monitor.
    """

    def __init__(self, name, matcher, stream=None):
        BaseHandler.__init__(self, name, matcher, stream=stream)
        self._payload = queue.Queue()

    def prerun(self, payload):
        """Store the matched stanza when received during processing.

        :param payload: The matched
            :class:`~sleekxmpp.xmlstream.stanzabase.ElementBase` object.
        """
        self._payload.put(payload)

    def run(self, payload):
        """Do not process this handler during the main event loop."""
        pass

    def wait(self, timeout=None):
        """Block an event handler while waiting for a stanza to arrive.

        Be aware that this will impact performance if called from a
        non-threaded event handler.

        Will return either the received stanza, or ``False`` if the
        waiter timed out.

        :param int timeout: The number of seconds to wait for the stanza
            to arrive. Defaults to the the stream's
            :class:`~sleekxmpp.xmlstream.xmlstream.XMLStream.response_timeout`
            value.
        """
        if timeout is None:
            timeout = self.stream().response_timeout

        elapsed_time = 0
        stanza = False
        while elapsed_time < timeout and not self.stream().stop.is_set():
            try:
                stanza = self._payload.get(True, 1)
                break
            except queue.Empty:
                elapsed_time += 1
                if elapsed_time >= timeout:
                    log.warning("Timed out waiting for %s", self.name)
        self.stream().remove_handler(self.name)
        return stanza

    def check_delete(self):
        """Always remove waiters after use."""
        return True



# jsb/utils/name.py
#
#

"""
name related helper functions.

google requirements on file names:
  - It must contain only letters, numbers, _, +, /, $, ., and -.
  - It must be less than 256 chars.
  - It must not contain "/./", "/../", or "//".
  - It must not end in "/".
  - All spaces must be in the middle of a directory or file name.


"""

## jsb imports

from ggz.utils.generic import toenc, fromenc
from ggz.lib.errors import NameNotSet

## basic imports

import string
import os
import re

## defines

allowednamechars = string.ascii_letters + string.digits + "-."

## slugify function taken from django (not used now)

def slugify(value):
    """
    Normalizes string, converts to lowercase, removes non-alpha characters,
    and converts spaces to hyphens.
    """
    import unicodedata
    value = unicodedata.normalize('NFKD', value)
    value = str(re.sub('[^\w\s-]', '', value).strip())
    return re.sub('[-\s]+', '-', value)

## stripname function

def stripname(namein, allowed=""):
    """ strip all not allowed chars from name. """
    if not namein: raise NameNotSet(namein)
    n = namein
    res = []
    allow = allowednamechars + allowed
    for c in n:
        if ord(c) < 31: continue
        elif c in allow: res.append(c)
        else: res.append("_" + str(ord(c)))
    return ''.join(res)

## testname function

def testname(name):
    """ test if name is correct. """
    for c in name:
        if c not in allowednamechars or ord(c) < 31: return False
    return True

def oldname(name):
    from ggz.lib.datadir import getdatadir
    if name.startswith("-"): name[0] = "+"
    name = name.replace("@", "+")
    if os.path.exists(getdatadir() + os.sep + name): return name
    name = name.replace("-", "#")
    name  = prevchan.replace("+", "@")
    if os.path.exists(getdatadir() + os.sep + name): return name
    return ""

## old jsonbot functions

oldallowednamechars = string.ascii_letters + string.digits + '_+/$.-'

## stripname function

def oldstripname(namein, allowed=""):
    """ strip all not allowed chars from name. """
    if not namein: raise NameNotSet(namein)
    n = namein.replace(os.sep, '+')
    n = n.replace("/", '+')
    n = n.replace("@", '+')
    n = n.replace("#", '-')
    n = n.replace("!", '.')
    res = []
    allow = oldallowednamechars + allowed
    for c in n:
        if ord(c) < 31: continue
        elif c in allow: res.append(c)
        else: res.append("-" + str(ord(c)))
    return ''.join(res)




# jsb/plugs/common/watcher.py
#
#

""" watch channels. channels events can be of remote origin. """

## jsb imports

from ggz.lib.commands import cmnds
from ggz.lib.callbacks import callbacks, remote_callbacks, last_callbacks, first_callbacks
from ggz.lib.persist import PlugPersist
from ggz.lib.fleet import getfleet
from ggz.utils.exception import handle_exception
from ggz.lib.examples import examples
from ggz.utils.locking import locked
from ggz.utils.generic import stripcolor
from ggz.lib.eventbase import EventBase

## plugin imports

from ggz.utils.format import formatevent

## basic imports

import copy
import logging

## defines

cpy = copy.deepcopy

## Watched class

class Watched(PlugPersist):

    """ Watched object contains channels and subscribers. """

    def __init__(self, filename):
        PlugPersist.__init__(self, filename)
        self.data.channels = self.data.channels or {}
        self.data.whitelist = self.data.whitelist or []
        self.data.descriptions = self.data.descriptions or {}

    def subscribe(self, botname, type, channel, jid):
        """ subscrive a jid to a channel. """ 
        channel = channel.lower()
        jid = str(jid)
        if channel not in self.data.channels: self.data.channels[channel] = []
        if not [botname, type, jid] in self.data.channels[channel]:
            self.data.channels[channel].append([botname, type, jid])
            self.save()
        return True

    def unsubscribe(self, botname, type, channel, jid):
        """ unsubscribe a jid from a channel. """ 
        channel = channel.lower()
        self.data.channels[channel].remove([botname, type, str(jid)])
        self.save()
        return True

    def reset(self, channel):
        """ unsubscribe a jid from a channel. """ 
        channel = channel.lower()
        self.data.channels[channel] = []
        self.save()
        return True

    def subscribers(self, channel):
        """ get all subscribers of a channel. """
        channel = channel.lower()
        try: return self.data.channels[channel]
        except KeyError: return []

    def check(self, channel):
        """ check if channel has subscribers. """
        channel = channel.lower()
        return channel in self.data.channels

    def enable(self, channel):
        """ add channel to whitelist. """
        channel = channel.lower()
        if not channel in self.data.whitelist:
            self.data.whitelist.append(channel)
            self.save()

    def disable(self, channel):
        """ remove channel from whitelist. """
        channel = channel.lower()
        try: self.data.whitelist.remove(channel)
        except ValueError: return False
        self.save()
        return True

    def available(self, channel):
        """ check if channel is on whitelist. """
        channel = channel.lower()
        return channel in self.data.whitelist

    def channels(self, channel):
        """ return channels on whitelist. """
        channel = channel.lower()
        res = []
        for chan, targets in self.data.channels.items():
            if channel in str(targets): res.append(chan)
        return res

## watched object

watched = Watched('channels')

## functions

bots = {}

def writeout(botname, type, channel, txt, eventjson):
    """ output the watched event to the channel. """
    event = EventBase().load(eventjson)
    global bots
    watchbot = None
    if botname in bots: watchbot = bots[botname]
    if not watchbot: watchbot = getfleet().byname(botname)
    if not watchbot: watchbot = getfleet().makebot(type, botname)
    if watchbot:
         if not botname in bots: bots[botname] = watchbot
         txt = watchbot.normalize(txt)
         txt = stripcolor(txt)
         watchbot.outnocb(channel, txt, event=event)

## callbacks

def prewatchcallback(bot, event):
    """ watch callback precondition. """
    if event.isdcc: return False
    logging.debug("watcher - checking %s - %s" % (event.channel, event.userhost))
    if not event.channel: return False
    if not event.txt: return
    return watched.check(str(event.channel)) and event.how != "background" and event.forwarded

def watchcallback(bot, event):
    """ the watcher callback, see if channels are followed and if so send data. """
    #if not event.allowwatch: logging.warn("watch - allowwatch is not set - ignoring %s" % event.userhost) ; return
    subscribers = watched.subscribers(event.channel)
    watched.data.descriptions[event.channel.lower()] = event.title
    logging.info("watcher - %s - %s" % (event.channel, str(subscribers)))
    for item in subscribers:
        try:
            try: (botname, type, channel) = item
            except ValueError: continue
            if not event.allowatch: pass
            elif channel not in event.allowwatch: logging.warn("watcher - allowwatch denied %s - %s" % (channel, event.allowwatch)) ; continue
            m = formatevent(bot, event, subscribers, True)
            if event.cbtype in ['OUTPUT', 'JOIN', 'PART', 'QUIT', 'NICK']: txt = "[!] %s" % m.txt
            else: txt = "[%s] %s" % (m.nick or event.nick or event.auth, m.txt)
            if txt.count('] [') > 2: logging.debug("watcher - %s - skipping %s" % (type, txt)) ; continue
            logging.warn("watcher - forwarding to %s" % channel)
            writeout(botname, type, channel, txt, event.tojson())
        except Exception as ex: handle_exception()

first_callbacks.add('BLIP_SUBMITTED', watchcallback, prewatchcallback)
first_callbacks.add('PRIVMSG', watchcallback, prewatchcallback)
first_callbacks.add('JOIN', watchcallback, prewatchcallback)
first_callbacks.add('PART', watchcallback, prewatchcallback)
first_callbacks.add('QUIT', watchcallback, prewatchcallback)
first_callbacks.add('NICK', watchcallback, prewatchcallback)
first_callbacks.add('OUTPUT', watchcallback, prewatchcallback)
first_callbacks.add('MESSAGE', watchcallback, prewatchcallback)
first_callbacks.add('CONSOLE', watchcallback, prewatchcallback)
first_callbacks.add('WEB', watchcallback, prewatchcallback)
first_callbacks.add('DISPATCH', watchcallback, prewatchcallback)
first_callbacks.add('TORNADO', watchcallback, prewatchcallback)

## watcher-start command

def handle_watcherstart(bot, event):
    """ arguments: [<channel>] - start watching a target (channel/wave). """
    target = event.rest or event.channel
    watched.subscribe(bot.cfg.name, bot.type, target, event.channel)
    if not target in event.chan.data.watched:
        event.chan.data.watched.append(target)
        event.chan.save()
    event.done()

cmnds.add('watcher-start', handle_watcherstart, 'OPER')
cmnds.add('watch', handle_watcherstart, 'USER')
examples.add('watcher-start', 'start watching a channel. ', 'watcher-start <channel>')

## watcher-reset command

def handle_watcherreset(bot, event):
    """ no arguments - reset all watcher for a channel. """
    watched.reset(event.channel)
    event.done()

cmnds.add('watcher-reset', handle_watcherreset, ['OPER', 'USER', 'GUEST'])
examples.add('watcher-reset', 'stop watching', 'watcher-reset')

## watcher-stop command

def handle_watcherstop(bot, event):
    """ arguments: [<channel>] - stop watching a channel. """
    if not event.rest: target = event.origin
    else: target = event.rest.strip()
    try: watched.unsubscribe(bot.cfg.name, bot.type, target, event.channel)
    except Exception as ex: event.reply("error removing watcher: %s" % str(ex)) ; return
    if target in event.chan.data.watched:
        event.chan.data.watched.remove(target)
        event.chan.save()
    event.done()

cmnds.add('watcher-stop', handle_watcherstop, ['OPER', 'USER', 'GUEST'])
examples.add('watcher-stop', 'stop watching a channel', 'watcher-stop #dunkbots')

## watcher-list command

def handle_watcherlist(bot, event):
    """ no arguments - see what channels we are watching. """
    chans = watched.channels(event.channel)
    if chans:
        res = []
        for chan in chans: 
            try: res.append(chan)
            except KeyError: res.append(chan)
        event.reply("channels watched on %s: " % event.channel, res)

cmnds.add('watcher-list', handle_watcherlist, ['OPER', 'USER', 'GUEST'])
examples.add('watcher-list', 'show what channels we are watching', 'watcher-list')

## watcher-subscribers command

def handle_watchersubscribers(bot, event):
    """" no arguments - show channels that are watching us. """
    event.reply("watchers for %s: " % event.channel, watched.subscribers(event.channel))

cmnds.add('watcher-subscribers', handle_watchersubscribers, ['OPER', 'USER', 'GUEST'])
examples.add('watcher-subscribers', 'show channels that are watching us. ', 'watcher-list')



#!/usr/bin/env python
#
# Copyright 2011 Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Implementation of platform-specific functionality.

For each function or class described in `tornado.platform.interface`,
the appropriate platform-specific implementation exists in this module.
Most code that needs access to this functionality should do e.g.::

    from tornado.platform.auto import set_close_exec
"""

import os

if os.name == 'nt':
    from tornado.platform.windows import set_close_exec, Waker
else:
    from tornado.platform.posix import set_close_exec, Waker



"""
    SleekXMPP: The Sleek XMPP Library
    Copyright (C) 2011 Nathanael C. Fritz, Dann Martens (TOMOTON).
    This file is part of SleekXMPP.

    See the file LICENSE for copying permission.
"""

from sleekxmpp.plugins.xep_0009.stanza.RPC import RPCQuery, MethodCall, MethodResponse



# jsb/plugs/socket/mpd.py
#
#
# interact with (your) Music Player Daemon
# (c) Wijnand 'tehmaze' Modderman - http://tehmaze.com
# BSD License
#
# CHANGELOG
#  2011-02-20
#   * sMiLe - changed methods to be similar to the mpc commands
#  2011-02-13
#   * sMiLe - added several new functions
#  2011-01-16
#   * BHJTW - adapted to jsonbot
#  2008-10-30
#   * fixed "Now playing" when having a password on MPD
#  2007-11-16
#   * added watcher support
#   * added formatting options ('song-status')
#   * added more precision to duration calculation
#  2007-11-10
#   * initial version
#
# REFERENCES
#
#  The MPD wiki is a great resource for MPD information, especially:
#   * http://mpd.wikia.com/wiki/MusicPlayerDaemonCommands
#

""" music player daemon control. """

__version__ = '2007111601'


## jsb imports

from ggz.lib.commands import cmnds
from ggz.lib.datadir import getdatadir
from ggz.lib.examples import examples
from ggz.lib.fleet import fleet
from ggz.utils.pdod import Pdod
from ggz.lib.persistconfig import PersistConfig
from ggz.lib.threads import start_new_thread

## basic imports

import os
import socket
import time

## defines

cfg = PersistConfig()
cfg.define('server-host', '127.0.0.1')
cfg.define('server-port', 6600)
cfg.define('server-pass', '')
cfg.define('socket-timeout', 15)
cfg.define('watcher-interval', 10)
cfg.define('watcher-enabled', 0)
cfg.define('song-status', 'now playing: %(artist)s - %(title)s on "%(album)s" (duration: %(time)s)')

## classes

class MPDError(Exception):
    """ exception to raise. """
    pass

class MPDDict(dict):
    """ return ? on no existant entry. """
    def __getitem__(self, item):
        if not item in self:
            return '?'
        else:
            return dict.__getitem__(self, item)

class MPDWatcher(Pdod):
    """ the MPD watcher. """
    def __init__(self):
        Pdod.__init__(self, os.path.join(getdatadir() + os.sep + 'plugs' + os.sep + 'ggz.plugs.sockets.mpd', 'mpd'))
        self.running = False
        self.lastsong = -1

    def add(self, bot, ievent):
        """ add a watcher. """
        if not self.has_key2(bot.cfg.name, ievent.channel):
            self.set(bot.cfg.name, ievent.channel, True)
            self.save()

    def remove(self, bot, ievent):
        """ remove a watcher. """
        if self.has_key2(bot.cfg.name, ievent.channel):
            del self.data[bot.cfg.name][ievent.channel]
            self.save()

    def start(self):
        """ start the watcher. """
        self.running = True
        start_new_thread(self.watch, ())

    def stop(self):
        """ stop the watcher. """
        self.running = False

    def watch(self):
        """ do the actual watching. """
        if not cfg.get('watcher-enabled'):
            raise MPDError('watcher not enabled, use "!%s-cfg watcher-enabled 1" to enable' % os.path.basename(__file__)[:-3])
        while self.running:
            if self.data:
                try:
                    status = MPDDict(mpd('currentsong'))
                    songid = int(status['id'])
                    if songid != self.lastsong:
                        self.lastsong = songid
                        self.announce(status)
                except MPDError:
                    pass
                except KeyError:
                    pass
            time.sleep(cfg.get('watcher-interval'))

    def announce(self, status):
        """ announce a new status. """
        if not self.running or not cfg.get('watcher-enabled'):
            return
        status['time'] = mpd_duration(status['time'])
        song = cfg.get('song-status') % status
        for name in list(self.data.keys()):
            bot = fleet.byname(name)
            if bot:
                for channel in list(self.data[name].keys()):
                    bot.say(channel, song)

## init

watcher = MPDWatcher()
if not watcher.data:
    watcher = MPDWatcher()

def init():
    if cfg.get('watcher-enabled'):
        watcher.start()
    return 1

def shutdown():
    if watcher.running:
        watcher.stop()
    return 1

## mpd-function

def mpd(command):
    """ do a MPD command. """
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.settimeout(cfg.get('socket-timeout'))
        s.connect((cfg.get('server-host'), cfg.get('server-port')))
    except socket.error as e:
        raise MPDError('Failed to connect to server: %s' % str(e))
    m = s.makefile('r')
    l = m.readline()
    if not l.startswith('OK MPD '): s.close() ; raise MPDError('Protocol error')
    if cfg.get('server-pass') and cfg.get('server-pass') != 'off':
        s.send(bytes('password %s\n' % cfg.get('server-pass'), "UTF-8"))
        l = m.readline()
        if not l.startswith('OK'):
            s.close()
            raise MPDError('Protocol error')
    s.send(bytes('%s\n' % command, "UTF-8"))
    s.send(bytes('close\n', "UTF-8"))
    d = []
    while True:
        l = m.readline().strip()
        if not l or l == 'OK':
            break
        if ': ' in l:
            l = l.split(': ', 1)
            l[0] = l[0].lower()
            d.append(tuple(l))
    s.close()
    return d

def mpd_duration(timespec):
    """ return duration string. """
    try:
        timespec = int(timespec)
    except ValueError:
        return 'unknown'
    timestr  = ''
    m = 60
    h = m * 60
    d = h * 24
    w = d * 7
    if timespec > w:
        w, timespec = divmod(timespec, w)
        timestr = timestr + '%02dw' % w
    if timespec > d:
        d, timespec = divmod(timespec, d)
        timestr = timestr + '%02dd' % d
    if timespec > h:
        h, timespec = divmod(timespec, h)
        timestr = timestr + '%02dh' % h
    if timespec > m:
        m, timespec = divmod(timespec, m)
        timestr = timestr + '%02dm' % m
    return timestr + '%02ds' % timespec

## mpd command

def handle_mpd(bot, ievent):
    """ no arguments - display mpd status. """
    try:
        result = []
        song = MPDDict(mpd('currentsong'))
        status = MPDDict(mpd('status'))
        status['time'] = mpd_duration(status['time'])
        reply = ''
        if status['state'] == 'play': 
            status['state'] = 'playing'
            reply += '%s: %s\n[%s] #%s/%s %s (%s)\n' % (song['name'], song['title'], status['state'], status['song'], status['playlistlength'], status['time'], '0')
        if status['state'] == 'stop': status['state'] = 'stopped'
        reply += 'volume: %s | repeat: %s | random: %s | single: %s | consume: %s' % (status['volume'], status['repeat'], status['random'], status['single'], status['consume'])
        ievent.reply(reply)
    except MPDError as e: ievent.reply(str(e))

#mpd                                           Display status
cmnds.add('mpd', handle_mpd, 'USER', threaded=True)
examples.add('mpd', 'Display mpd status', 'mpd')

## mpd-outputs command

def handle_mpd_outputs(bot, ievent):
    """ show mpd outputs. """
    # Output 1 (My ALSA Device) is enabled
    # [('outputid', '0'), ('outputname', 'My ALSA Device'), ('outputenabled', '1')]
    try:
        outputs = mpd('outputs')
        outputid = '?'
        outputname = '?'
        outputenabled = '?'
        result = []
        for item in outputs:
            if item[0] == 'outputid': outputid = int(item[1])+1
            if item[0] == 'outputname': outputname = item[1]
            if item[0] == 'outputenabled':
                if item[1] == '1': outputenabled = 'enabled' 
                else: outputenabled = 'disabled'
                result.append('Output %d (%s) is %s' % (outputid, outputname, outputenabled))
                outputid = '?'
                outputname = '?'
                outputenabled = '?'
        ievent.reply("\n".join(result))
    except MPDError as e:
        ievent.reply(str(e))

## mpd-enable command

def handle_mpd_enable(bot, ievent):
    """ arguments: <outputnumber> - enable an output. """
    try:
        try: output = int(ievent.args[0])-1
        except: ievent.missing('<output #>') ; return
        result = mpd('enableoutput %d' % output)
        ievent.reply(result)
    except MPDError as e:
        ievent.reply(str(e))

## mpd-disable command

def handle_mpd_disable(bot, ievent):
    """ arguments: <outputnumber> - disable an output. """
    try:
        try: output = int(ievent.args[0])-1
        except: ievent.missing('<output #>') ; return
        result = mpd('disableoutput %d' % output)
        ievent.reply(result)
    except MPDError as e:
        ievent.reply(str(e))

## mpd-playlist command

def handle_mpd_playlist(bot, ievent):
    """ no arguments - show playlist. """
    try:
        playlist = mpd('playlistinfo')
        tmp = ''
        result = []
        for item in playlist:
           if item[0] == 'file': 
              if not tmp == '': result.append(tmp)
              tmp = item[1]
           if item[0] == 'title': tmp = item[1]
           if item[0] == 'name': tmp = '%s: %s' % (item[1], tmp)
        ievent.reply("\n".join(result))
    except MPDError as e:
        ievent.reply(str(e))

## mpd-lsplaylists command

def handle_mpd_lsplaylists(bot, ievent):
    """ no arguments - show playlists. """
    try:
        playlists = mpd('lsinfo')
        result = []
        for item in playlists:
           if item[0] == 'playlist': 
              result.append(item[1])
        ievent.reply("\n".join(result))
    except MPDError as e:
        ievent.reply(str(e))

## mpd-playlistmanipulation command

def handle_mpd_playlist_manipulation(bot, ievent, command):
    """ arguments: <playlist> - manipulate a playlist, the command is given as parameter """
    try:
        if not ievent.args:
            ievent.missing('<playlist>')
            return
        playlist = str(ievent.args[0])
        result = mpd('%s %s' % (command, playlist))
        ievent.reply('Playlist %s loaded.' % playlist)
    except MPDError as e:
        ievent.reply(str(e))

## mpd-load command

def handle_mpd_load(bot, ievent):
     """ arguments: <playlist> - load playlist. """
     handle_mpd_playlist_manipulation(bot, ievent, 'load')

## mpd-save command

def handle_mpd_save(bot, ievent):
     """ arguments: <playlist> - save playlist. """
     handle_mpd_playlist_manipulation(bot, ievent, 'save')

## mpd-rm command

def handle_mpd_rm(bot, ievent):
     """ arguments: <playlist> - remove playlist. """
     handle_mpd_playlist_manipulation(bot, ievent, 'rm')

## mpd-np command

def handle_mpd_np(bot, ievent):
    """ no arguments - show current playing song. """
    try:
        status = MPDDict(mpd('currentsong'))
        status['time'] = mpd_duration(status['time'])
        ievent.reply(cfg.get('song-status') % status)
    except MPDError as e:
        ievent.reply(str(e))

#mpd-current                                   Show the currently playing song
cmnds.add('mpd-current',    handle_mpd_np,    'USER', threaded=True)
cmnds.add('mpd-np',    handle_mpd_np,    'USER', threaded=True)
examples.add('mpd-current', 'Show the currently playing song', 'mpd-current')

## mpd-simpleseek command

def handle_mpd_simple_seek(bot, ievent, command):
    """ no arguments - do a seek. """
    try:
        mpd(command)
        #handle_mpd_np(bot, ievent)
        handle_mpd(bot, ievent)
        ievent.done()
    except MPDError as e:
        ievent.reply(str(e))

## mpd-next command

handle_mpd_next = lambda b,i: handle_mpd_simple_seek(b,i,'next') 

#mpd-next                                      Play the next song in the current playlist
cmnds.add('mpd-next',  handle_mpd_next,  'MPD', threaded=True)
examples.add('mpd-next', 'play the next song in the current playlist', 'mpd-next')

## mpd-prev command

handle_mpd_prev = lambda b,i: handle_mpd_simple_seek(b,i,'prev') 

#mpd-prev                                      Play the previous song in the current playlist
cmnds.add('mpd-prev',  handle_mpd_prev,  'MPD', threaded=True)
examples.add('mpd-prev', 'play the previous song in the current playlist', 'mpd-prev')

## mpd-play command

handle_mpd_play = lambda b,i: handle_mpd_simple_seek(b,i,'play') 

#mpd-play [<position>]                         Start playing at <position> (default: 1)
cmnds.add('mpd-play',  handle_mpd_play,  'MPD', threaded=True)
examples.add('mpd-play', 'start playing at <position> (default: 1)', 'mpd-play')

## mpd-stop command

handle_mpd_stop = lambda b,i: handle_mpd_simple_seek(b,i,'stop') 

## mpd-pause command

handle_mpd_pause = lambda b,i: handle_mpd_simple_seek(b,i,'pause') 

#mpd-pause                                     Pauses the currently playing song
cmnds.add('mpd-pause', handle_mpd_pause, 'MPD', threaded=True)
examples.add('mpd-pause', 'pauses the currently playing song', 'mpd-pause')

## mpd-clear command

handle_mpd_clear = lambda b,i: handle_mpd_simple_seek(b,i,'clear') 

## mpd-crop command

handle_mpd_crop = lambda b,i: handle_mpd_simple_seek(b,i,'crop') 

#mpd-crop                                      Remove all but the currently playing song
cmnds.add('mpd-crop', handle_mpd_crop, 'MPD', threaded=True)
examples.add('mpd-crop', 'remove all but the currently playing song', 'mpd-crop')

## mpd-shuffle command

handle_mpd_shuffle = lambda b,i: handle_mpd_simple_seek(b,i,'shuffle') 

## mpd-repeat command

handle_mpd_repeat = lambda b,i: handle_mpd_toggle_option(b,i,'repeat') 

## mpd-random command

handle_mpd_random = lambda b,i: handle_mpd_toggle_option(b,i,'random') 

## mpd-single command

handle_mpd_single= lambda b,i: handle_mpd_toggle_option(b,i,'single') 

## mpd-consume command

handle_mpd_consume= lambda b,i: handle_mpd_toggle_option(b,i,'consume') 

## mpd-crossfade command

handle_mpd_crossfade= lambda b,i: handle_mpd_set_option(b,i,'crossfade') 

## mpd-find command

def handle_mpd_find(bot, ievent):
    type = 'title'
    args = ievent.args
    if args and args[0].lower() in ['title', 'album', 'artist']: type = args[0].lower() ; args = args[1:]
    if not args: ievent.missing('[<type>] <what>') ; return
    try:
        find = mpd('search %s "%s"' % (type, ' '.join(args)))
        show = []
        for item, value in find:
            if item == 'file': show.append(value)
        if show: ievent.reply("results: ", show)
        else: ievent.reply('no result')
    except MPDError as e: ievent.reply(str(e))

## mpd-add command

def handle_mpd_add(bot, ievent):
    """ arguments: <filename> - add a file to the MPD. """
    if not ievent.args:
        ievent.missing('<filename>')
        return
    try:
        addid = MPDDict(mpd('addid "%s"' % ievent.rest))
        if 'id' not in addid:
            ievent.reply('failed to load song "%s"' % ievent.rest)
        else:
            ievent.reply('added song with id "%s", use "mpd-jump %s" to start playback' % (addid['id'], addid['id']))
    except MPDError as e:
        ievent.reply(str(e))

#mpd-add <file>                                Add a song to the current playlist
cmnds.add('mpd-add', handle_mpd_add, 'MPD', threaded=True)
cmnds.add('mpd-queue', handle_mpd_add, 'MPD', threaded=True)
examples.add('mpd-add', 'add a song to the current playlist', 'mpd-add mp3/bigbeat/fatboy slim/fatboy slim - everybody needs a 303.mp3')

## mpd-del command

def handle_mpd_del(bot, ievent):
    """ arguments: <position> - remove a song from the current playlist. """
    if not ievent.args: ievent.missing('<position>') ; return
    try: result = mpd('delete %d' % int(ievent.args[0])) ; ievent.reply(result)
    except MPDError as e: ievent.reply(str(e))

#mpd-del <position>                            Remove a song from the current playlist
cmnds.add('mpd-del',  handle_mpd_del,  'MPD', threaded=True)
examples.add('mpd-del', 'remove a song from the current playlist', 'mpd-del 1')

## mpd-jump command

def handle_mpd_jump(bot, ievent):
    """ arguments: <id> - jump to playlist. """
    pos = 0
    try:    pos = int(ievent.args[0])
    except: pass
    if not pos: ievent.missing('<playlist id>') ; return
    try:
        mpd('playid %d' % pos)
        handle_mpd_np(bot, ievent)
    except MPDError as e: ievent.reply(str(e))

cmnds.add('mpd-jump',  handle_mpd_jump, 'MPD', threaded=True)
examples.add('mpd-jump', 'jump to the specified playlist id', 'mpd-jump 777')

## mpd-stats command

def handle_mpd_stats(bot, ievent):
    """ no arguments - determine MPD stats. """
    try:
        status = MPDDict(mpd('stats'))
        status['total playtime'] = mpd_duration(status['playtime'])
        status['total database playtime'] = mpd_duration(status['db_playtime'])
        status['uptime'] = mpd_duration(status['uptime'])
        del status['playtime']
        del status['db_playtime']
        del status['db_update']
        result = []
        for item in sorted(status.keys()):
            result.append('%s: %s' % (item, status[item]))
        ievent.reply(" | ".join(result))
    except MPDError as e:
        ievent.reply(str(e))

#mpd-stats                                     Display statistics about MPD
cmnds.add('mpd-stats', handle_mpd_stats, 'USER', threaded=True)
examples.add('mpd-stats', 'Display statistics about MPD', 'mpd-stats')

## mpd-volume command

def handle_mpd_volume(bot, ievent):
    """ arguments: <volume> - set MPD volume. """
    volume = 0
    try:    volume = int(ievent.args[0])
    except: pass
    if not volume:
        status = MPDDict(mpd('status'))
        ievent.reply('Current volume: %s' % status['volume'])
        return        
    try:
        mpd('setvol %d' % volume)
        ievent.reply('Volume set to %d' % volume)
    except MPDError as e:
        ievent.reply(str(s))

## mpd-toggleoption command

def handle_mpd_toggle_option(bot, ievent, option):
    """ arguments: <value> - toggle variable. """
    if ievent.args:
        val = 'on'
        try:    val = ievent.args[0]
        except: pass
        if val == '0' or val == 'off': val = '0'
        else: val = '1'
    else:
        status = MPDDict(mpd('status'))
        if status[option] == '0': val = '1'
        else: val = '0'
    try:
        mpd('%s %s' % (option, val))
        handle_mpd(bot, ievent)
    except MPDError as e:
        ievent.reply(str(e))

## mpd-setoption command

def handle_mpd_set_option(bot, ievent, option):
    """ arguments: <value> - set value of an option. """ 
    try:
        if ievent.args:
            val = -1
            try:    val = int(ievent.args[0])
            except: pass
            if val > 0: 
                mpd('%s %s' % (option, val))
            
            else: 
                ievent.reply('"off" is not 0 or positive integer' % ievent.args[0])
                return
        else:
            status = MPDDict(mpd('status'))
            ievent.reply("%s %s" % (option, status['xfade']))
    except MPDError as e:
        ievent.reply(str(e))

## mpd-watchstart command

def handle_mpd_watch_start(bot, ievent):
    """ no arguments - start MPD watcher. """
    if not cfg.get('watcher-enabled'):
        ievent.reply('watcher not enabled, use "!%s-cfg watcher-enabled 1" to enable and reload the plugin' % os.path.basename(__file__)[:-3])
        return
    watcher.add(bot, ievent)
    ievent.reply('ok')

cmnds.add('mpd-watch-start', handle_mpd_watch_start, 'MPD', threaded=True)

## mpd-watchstop command

def handle_mpd_watch_stop(bot, ievent):
    """ no arguments - stop MPD watcher. """
    if not cfg.get('watcher-enabled'):
        ievent.reply('watcher not enabled, use "!%s-cfg watcher-enabled 1" to enable and reload the plugin' % os.path.basename(__file__)[:-3])
        return
    watcher.remove(bot, ievent)
    ievent.reply('ok')

cmnds.add('mpd-watch-stop',  handle_mpd_watch_stop, 'MPD', threaded=True)

## mpd-watchlist command

def handle_mpd_watch_list(bot, ievent):
    """ no aruguments - list the watchers channels. """
    if not cfg.get('watcher-enabled'):
        ievent.reply('watcher not enabled, use "!%s-cfg watcher-enabled 1" to enable and reload the plugin' % os.path.basename(__file__)[:-3])
        return
    result = []
    for name in sorted(watcher.data.keys()):
        if watcher.data[name]:
            result.append('on %s:' % name)
        for channel in sorted(watcher.data[name].keys()):
            result.append(channel)
    if result:
        ievent.reply(' '.join(result))
    else:
        ievent.reply('no watchers running')

cmnds.add('mpd-watch-list',  handle_mpd_watch_list, 'MPD', threaded=True)

## register command and exmaples

#mpd-toggle                                    Toggles Play/Pause, plays if stopped
cmnds.add('mpd-toggle', handle_mpd_pause, 'MPD', threaded=True)
examples.add('mpd-toggle', 'Toggles Play/Pause, plays if stopped', 'mpd-toggle')

#mpd-stop                                      Stop the currently playing playlists
cmnds.add('mpd-stop',  handle_mpd_stop,  'MPD', threaded=True)
examples.add('mpd-stop', 'Stop the currently playing playlists', 'mpd-stop')

# TODO mpd-seek [+-][HH:MM:SS]|<0-100>%              Seeks to the specified position
# cmnds.add('mpd-seek', handle_mpd_seek, 'MPD')
#mpd-clear                                     Clear the current playlist
cmnds.add('mpd-clear', handle_mpd_clear, 'MPD', threaded=True)
examples.add('mpd-clear', 'Clear the current playlist', 'mpd-clear')

#mpd-outputs                                   Show the current outputs
cmnds.add('mpd-outputs', handle_mpd_outputs, 'MPD', threaded=True)
examples.add('mpd-outputs', 'Show the current outputs', 'mpd-outputs')

#mpd-enable <output #>                         Enable a output
cmnds.add('mpd-enable', handle_mpd_enable, 'MPD', threaded=True)
examples.add('mpd-enable', 'Enable a output', 'mpd-enable <output #>')

#mpd-disable <output #>                        Disable a output
cmnds.add('mpd-disable', handle_mpd_disable, 'MPD', threaded=True)
examples.add('mpd-disable', 'Disable a output', 'mpd-disable <output #>')

#mpd-shuffle                                   Shuffle the current playlist
cmnds.add('mpd-shuffle', handle_mpd_shuffle, 'MPD', threaded=True)
examples.add('mpd-shuffle', 'Shuffle the current playlist', 'mpd-shuffle')

# TODO mpd move <from> <to>                          Move song in playlist
#cmnds.add('mpd-move', handle_mpd_move, 'MPD')
#examples.add('mpd-move', 'Move song in playlist', 'mpd-move <from> <to>')

#mpd-playlist                                  Print the current playlist
cmnds.add('mpd-playlist', handle_mpd_playlist, 'USER', threaded=True)
examples.add('mpd-playlist', 'Print the current playlist', 'mpd-playlist')

# TODO mpd listall [<file>]                          List all songs in the music dir
#cmnds.add('mpd-listall', handle_mpd_listall, 'USER')
#examples.add('mpd-listall', 'List all songs in the music dir', 'mpd-listall [<file>]')

# TODO mpd ls [<directory>]                          List the contents of <directory>
#cmnds.add('mpd-ls', handle_mpd_ls, 'USER')
#examples.add('mpd-ls', 'List the contents of <directory>', 'mpd-ls [<directory>]')

#mpd-lsplaylists                               List currently available playlists
cmnds.add('mpd-lsplaylists', handle_mpd_lsplaylists, 'USER', threaded=True)
examples.add('mpd-lsplaylists', 'List currently available playlists', 'mpd-lsplaylists')

#mpd-load <file>                               Load <file> as a playlist
cmnds.add('mpd-load', handle_mpd_load, 'MPD', threaded=True)
examples.add('mpd-load', 'Load <file> as a playlist', 'mpd-load <file>')

#mpd-save <file>                               Save a playlist as <file>
cmnds.add('mpd-save', handle_mpd_save, 'MPD', threaded=True)
examples.add('mpd-save', 'Save a playlist as <file>', 'mpd-save <file>')

#mpd-rm <file>                                 Remove a playlist
cmnds.add('mpd-rm', handle_mpd_rm, 'MPD', threaded=True)
examples.add('mpd-rm', 'Remove a playlist', 'mpd-rm <file>')

#mpd-volume [+-]<num>                          Set volume to <num> or adjusts by [+-]<num>
# TODO [+-]
cmnds.add('mpd-volume', handle_mpd_volume, 'MPD', threaded=True)
examples.add('mpd-volume', 'Set volume to <num> or adjusts by [+-]<num>', 'mpd-volume 42')

#mpd-repeat <on|off>                           Toggle repeat mode, or specify state
cmnds.add('mpd-repeat', handle_mpd_repeat, 'MPD', threaded=True)
examples.add('mpd-volume', 'Toggle repeat mode, or specify state', 'mpd-repeat off')

#mpd-random <on|off>                           Toggle random mode, or specify state
cmnds.add('mpd-random', handle_mpd_random, 'MPD', threaded=True)
examples.add('mpd-random', 'Toggle random mode, or specify state', 'mpd-random <on|off>')

#mpd-single <on|off>                           Toggle single mode, or specify state
cmnds.add('mpd-single', handle_mpd_single, 'MPD', threaded=True)
examples.add('mpd-single', 'Toggle single mode, or specify state', 'mpd-single <on|off>')

#mpd-consume <on|off>                          Toggle consume mode, or specify state
cmnds.add('mpd-consume', handle_mpd_consume, 'MPD', threaded=True)
examples.add('mpd-consume', 'Toggle consume mode, or specify state', 'mpd-consume <on|off>')

# TODO mpd search <type> <query>                     Search for a song
#cmnds.add('mpd-search', handle_mpd_search, 'MPD')
#examples.add('mpd-search', 'Search for a song', 'mpd-search <type> <query>')

#mpd-find <type> <query>                       Find a song (exact match)
cmnds.add('mpd-find',  handle_mpd_find,  'MPD', threaded=True)
examples.add('mpd-find', 'Find a song (exact match)', 'mpd-find title love')

# TODO mpd-findadd <type> <query>                    Find songs and add them to the current playlist
# cmnds.add('mpd-findadd', handle_mpd_findadd, 'MPD')
#examples.add('mpd-findadd', 'Find songs and add them to the current playlist', 'mpd-findadd <type> <query>')

# TODO mpd-list <type> [<type> <query>]              Show all tags of <type>
# cmnds.add('mpd-list', handle_mpd_list, 'MPD')
#examples.add('mpd-list', 'Show all tags of <type>', 'mpd-list <type> [<type> <query>]')

#mpd-crossfade [<seconds>]                     Set and display crossfade settings
cmnds.add('mpd-crossfade', handle_mpd_crossfade, 'MPD', threaded=True)
examples.add('mpd-crossfade', 'Set and display crossfade settings', 'mpd-crossfade 42')

#mpd-update [<path>]                           Scan music directory for updates
#cmnds.add('mpd-update', handle_mpd_update, 'MPD')

#mpd-sticker <uri> <get|set|list|del> <args..> Sticker management
#cmnds.add('mpd-sticker', handle_mpd_sticker, 'MPD')


#mpd-version                                   Report version of MPD
#cmnds.add('mpd-version', handle_mpd_version, 'USER')

#mpd-idle [events]                             Idle until an event occurs
#cmnds.add('mpd-idle', handle_mpd_idle, 'MPD')

#mpd-idleloop [events]                         Continuously idle until an event occurs
#cmnds.add('mpd-idleloop', handle_mpd_idleloop, 'MPD')

# TODO mpd-replaygain [off|track|ablum]              Set or display the replay gain mode
# cmnds.add('mpd-replaygain', handle_mpd_replaygain, 'MPD')
#examples.add('mpd-replaygain', 'Set or display the replay gain mode', 'mpd-replaygain')




from ggz.lib.commands import cmnds
from ggz.plugs.extra.tinyurl import get_tinyurl
from urllib.parse import quote, unquote

def handle_lmgt(bot, ievent):
    """ google something for them; syntax: lmgt [search terms] """
    if len(ievent.args) < 1:
        ievent.reply("syntax: lmgt [search terms]")
        return

    a = "http://lmgtfy.com/?q=%s" % quote(" ".join(ievent.args))
    ievent.reply("Let me google that for you: %s" % get_tinyurl(a)[0])

cmnds.add("lmgt", handle_lmgt, ["OPER", "USER", "GUEST"])




# jsb/plugs/socket/chatlog.py
#
#

""" log channels to [hour:min] <nick> txt format, only logging to files is supported right now.  """

## jsb imports

from ggz.lib.commands import cmnds
from ggz.lib.callbacks import callbacks, remote_callbacks, last_callbacks, first_callbacks
from ggz.lib.persistconfig import PersistConfig
from ggz.utils.locking import lockdec
from ggz.utils.timeutils import hourmin
from ggz.lib.examples import examples
from ggz.utils.exception import handle_exception
from ggz.utils.lazydict import LazyDict
from ggz.lib.datadir import getdatadir
from ggz.utils.name import stripname
from ggz.utils.url import striphtml
from ggz.utils.format import formatevent, format_opt
from ggz.utils.log import init
from ggz.utils.statdict import StatDict
from ggz.utils.timeutils import striptime, strtotime2

## basic imports

import time
import os
import logging
import _thread
from os import path
from datetime import datetime

## locks

outlock = _thread.allocate_lock()
outlocked = lockdec(outlock)

## defines

cfg = PersistConfig()
cfg.define('channels', [])
cfg.define('format', 'log')
cfg.define('basepath', getdatadir())
cfg.define('nologprefix', '[nolog]')
cfg.define('nologmsg', '-= THIS MESSAGE NOT LOGGED =-')
cfg.define('backend', 'log')

logfiles = {}
backends = {}
stopped = False
db = None
eventstolog = ["OUTPUT", "PRIVMSG", "CONSOLE", "PART", "JOIN", "QUIT", "PRESENCE", "MESSAGE", "NOTICE", "MODE", "TOPIC", "KICK", "CONVORE", "TORNADO"]

## logging part

# BHJTW 21-02-2011 revamped to work with standard python logger

loggers = {}

def initlog(d):
    """ create the necesary directories to enable logging. """
    try: LOGDIR = d + os.sep + "chatlogs"
    except ImportError: LOGDIR = d + os.sep + "chatlogs"

    try:
        ddir = os.sep.join(LOGDIR.split(os.sep)[:-1])
        if not os.path.isdir(ddir): os.mkdir(ddir)   
    except: pass  

    try:
        if not os.path.isdir(LOGDIR): os.mkdir(LOGDIR)
    except: pass
    return LOGDIR

format = "%(message)s"

def timestr(dt):
    """ convert datatime object to a time string. """
    return dt.strftime(format_opt('timestamp_format'))   

## enablelogging function

def enablelogging(botname, channel):
    """ set loglevel to level_name. """
    global loggers
    LOGDIR = initlog(getdatadir())
    logging.warn("enabling on (%s,%s)" % (botname, channel))
    channel = stripname(channel)
    logname = "%s_%s" % (botname, channel)
    #if logname in loggers: logging.warn("there is already a logger for %s" % logname) ; return
    try:
        filehandler = logging.handlers.TimedRotatingFileHandler(LOGDIR + os.sep + logname + ".log", 'midnight')
        formatter = logging.Formatter(format)
        filehandler.setFormatter(formatter)  
    except IOError:
        filehandler = None
    chatlogger = logging.getLoggerClass()(logname)
    chatlogger.setLevel(logging.INFO)
    if chatlogger.handlers:
        for handler in chatlogger.handlers: chatlogger.removeHandler(handler)
    if filehandler: chatlogger.addHandler(filehandler) ; logging.warn("%s - logging enabled on %s" % (botname, channel))
    else: logging.error("no file handler found - not enabling logging.")
    global lastlogger
    lastlogger = chatlogger
    loggers[logname] = lastlogger

## do tha actual logging

@outlocked
def write(m): 
    """
      m is a dict with the following properties:
      datetime
      type : (comment, nick, topic etc..)
      target : (#channel, bot etc..)
      txt : actual message
      network
    """
    backend_name = cfg.get('backend', 'log')
    backend = backends.get(backend_name, log_write)
    if m.txt.startswith(cfg.get('nologprefix')): m.txt = cfg.get('nologmsg')
    backend(m)

def log_write(m):
    if stopped: return
    logname = "%s_%s" % (m.botname, stripname(m.target))
    timestamp = timestr(m.datetime)
    m.type = m.type.upper()
    line = '%(timestamp)s%(separator)s %(txt)s\n'%({
        'timestamp': timestamp, 
        'separator': format_opt('separator'),
         'nick': m.nick,
        'txt': m.txt,
        'nick': m.nick,
        'type': m.type
    })
    global loggers
    try: loggers[logname].info(line.strip())
    except KeyError: logging.error("no logger available for channel %s" % logname)
    except Exception as ex: handle_exception()

backends['log'] = log_write

## log function

def log(bot, event):
    """ format an event and send it to the logging backend. """
    m = formatevent(bot, event, cfg.get("channels") or [])
    if m["txt"]: write(m)

## chatlog precondition

def prechatlogcb(bot, ievent):
    """
        Check if event should be logged.  QUIT and NICK are not channel
        specific, so we will check each channel in log().

    """
    if not ievent.channel: logging.debug("channel not set .. not logging.") ; return False
    if not cfg.channels: logging.debug("no channels set") ; return False
    if [bot.cfg.name, ievent.channel] in cfg.get('channels'): logging.debug("%s %s in channels .. logging" % (bot.cfg.name, ievent.channel)) ; return True
    if not ievent.cbtype in eventstolog: logging.debug("%s ut not in eventstolog list." % ievent.cbtype) ; return False
    if ievent.msg: logging.debug("is messsage .. not logging") ; return False
    if ievent.cmnd in ('QUIT', 'NICK'): return True
    if ievent.cmnd == 'NOTICE':
        if [bot.cfg.name, ievent.arguments[0]] in cfg.get('channels'): return True
    logging.debug("not match for logging.")
    return False

## chatlog callbacks

def chatlogcb(bot, ievent):
    """ logging callback. """
    log(bot, ievent)

## plugin-start

def init():
    """ called upon plugin registration. """
    global stopped
    stopped = False
    global loggers
    for (botname, channel) in cfg.get("channels"):
        enablelogging(botname, channel)  
    callbacks.add("PRIVMSG", chatlogcb, prechatlogcb)
    callbacks.add("JOIN", chatlogcb, prechatlogcb)
    callbacks.add("PART", chatlogcb, prechatlogcb)
    callbacks.add("NOTICE", chatlogcb, prechatlogcb)
    callbacks.add("QUIT", chatlogcb, prechatlogcb)
    callbacks.add("NICK", chatlogcb, prechatlogcb)
    callbacks.add("PRESENCE", chatlogcb, prechatlogcb)
    callbacks.add("MESSAGE", chatlogcb, prechatlogcb)
    callbacks.add("CONSOLE", chatlogcb, prechatlogcb)
    callbacks.add("CONVORE", chatlogcb, prechatlogcb)
    first_callbacks.add("OUTPUT", chatlogcb, prechatlogcb)
    return 1

## plugin-stop

def shutdown():
    """ shutdown the plugin. """
    global stopped
    stopped = True
    for file in list(logfiles.values()):
        file.close()
    return 1

## chatlog-on command

def handle_chatlogon(bot, ievent):
    """ no arguments - enable chatlog. """
    chan = ievent.channel
    enablelogging(bot.cfg.name, chan)
    if [bot.cfg.name, chan] not in cfg.get('channels'):
        cfg['channels'].append([bot.cfg.name, chan])
        cfg.save()
    ievent.reply('chatlog enabled on (%s,%s)' % (bot.cfg.name, chan))

cmnds.add('chatlog-on', handle_chatlogon, 'OPER')
examples.add('chatlog-on', 'enable chatlog on the channel the commands is given in', 'chatlog-on')

## chatlog-off command

def handle_chatlogoff(bot, ievent):
    """ no arguments - disable chatlog. """
    try: cfg['channels'].remove([bot.cfg.name, ievent.channel]) ; cfg.save()
    except ValueError: ievent.reply('chatlog is not enabled in (%s,%s)' % (bot.cfg.name, ievent.channel)) ; return
    try: del loggers["%s-%s" % (bot.cfg.name, stripname(ievent.channel))]
    except KeyError: pass
    except Exception as ex: handle_exception()
    ievent.reply('chatlog disabled on (%s,%s)' % (bot.cfg.name, ievent.channel))

cmnds.add('chatlog-off', handle_chatlogoff, 'OPER')
examples.add('chatlog-off', 'disable chatlog on the channel the commands is given in', 'chatlog-off')

## chatlog-searh command

def handle_chatlogsearch(bot, event):
    """ arguments: <searchtxt> - search in the logs. """
    if not event.rest: event.missing("<searchtxt>") ; return
    result = []
    chatlogdir = getdatadir() + os.sep + "chatlogs"
    if event.options and event.options.channel: chan = event.options.channel
    else: chan = event.channel
    logs = os.listdir(chatlogdir)
    logs.sort()
    for f in logs:
        filename = stripname(f)
        if not chan[1:] in filename: continue
        for line in open(chatlogdir + os.sep + filename, 'r'):
            if event.rest in line: result.append(line)
    if result: event.reply("search results for %s: " % event.rest, result, dot= " || ")
    else: event.reply("no result found for %s" % chan)

cmnds.add("chatlog-search", handle_chatlogsearch, ["OPER", "USER", "GUEST"], threaded=True)
examples.add("chatlog-search", "search the chatlogs of a channel.", "chatlog-search jsonbot")

## chatlog-stats command

def handle_chatlogstats(bot, event):
    """ no arguments - create log stats of the channel, possible options: --chan <channel> """
    what = event.rest.strip()
    chatlogdir = getdatadir() + os.sep + "chatlogs"
    if event.options and event.options.channel: chan = event.options.channel
    else: chan = event.channel
    logs = os.listdir(chatlogdir)
    if not logs: event.reply("no logs available for %s" % chan) ; return
    now = time.time()
    if what: timetarget = strtotime2(what) ; what = striptime(what)
    else: timetarget = 0 ; what = None
    event.reply("creating stats for channel %s (%s)" % (chan, time.ctime(timetarget)))
    userstats = StatDict()
    wordstats = StatDict()
    stop = False
    for f in logs[::-1]:
        filename = stripname(f)
        channel = stripname(chan[1:])
        if not channel in filename: continue
        for line in open(chatlogdir + os.sep + filename, 'r'):
            splitted = line.strip().split()
            if len(splitted) < 2: continue
            who = "unknown"
            for i in splitted:
               if i.startswith("<"): who = i[1:-1]
            if what and who != what: continue
            timestr = "%s %s" % (splitted[0], splitted[1])
            logtime = strtotime2(timestr)
            if logtime:
                if logtime > timetarget: userstats.upitem(who)
                else: continue
            else: userstats.upitem(who)
            for word in splitted[4:]: wordstats.upitem(word)
    if what: result = wordstats.top()
    else: result = userstats.top()
    if result:
        res = ["%s: %s" % item for item in result]
        event.reply("stat results for %s: " % (what or chan), res)
    else: event.reply("no result found for %s" % (what or chan))

cmnds.add("chatlog-stats", handle_chatlogstats, ["OPER", "USER", "GUEST"], threaded=True)
examples.add("chatlog-stats", "stats of a channel.", "chatlog-stats")



# jsb/utils/id.py
#
#

""" id related functions. """

## jsb imports

from ggz.utils.generic import toenc

## basic imports

import uuid

## getrssid function

def getrssid(url, time):
    """ get an id based on url and time. """
    key = str(url) + str(time)
    return str(uuid.uuid3(uuid.NAMESPACE_DNS, toenc(key)))



# jsb/plugs/socket/confluence.py
#
#

"""
confluence.py - jsonbot module for performing lookups on a confluence server
Copyright 2011, Richard Bateman

Special thanks to Sean B. Palmer for his phenny module; many of the ideas for
this were adapted from that plugin

http://inamidst.com/phenny/
"""

## jsb imports

from ggz.lib.callbacks import callbacks
from ggz.lib.commands import cmnds
from ggz.lib.persist import PlugPersist
from ggz.lib.examples import examples
from ggz.plugs.extra.tinyurl import get_tinyurl

## basic imports

import logging
import xmlrpc.client
import re
import time

#import modules.activecollab

## defines

rpc_clients = {}
cfg = PlugPersist('confluence', {})

## fetRpcClient function

def getRpcClient(sInfo):
    if sInfo["name"] not in rpc_clients:
        base_url = "%s/rpc/xmlrpc" % sInfo["url"]
        server = xmlrpc.client.ServerProxy(base_url).confluence1

        username, password = sInfo["username"], sInfo["password"]
        auth = server.login(username, password)

        rpc_clients[sInfo["name"]] = (server, auth)

    return rpc_clients[sInfo["name"]]

## confluence-add_confluence_server command

def handle_add_confluence_server(bot, ievent):
    """ configure a new confluence server; syntax: add_confluence_server [server name] [url] [username] [password] """
    if len(ievent.args) != 4:
        ievent.reply("syntax: add_confluence_server [server name] [url] [username] [password]")
        return

    server = {
        "name": ievent.args[0],
        "url": ievent.args[1].strip("/"),
        "username": ievent.args[2],
        "password": ievent.args[3],
        "channels": {},
        "serverInfo": {},
    }

    if "servers" not in cfg.data:
        cfg.data["servers"] = {}
    cfg.data["servers"][server["name"]] = server
    cfg.save()

    ievent.reply("Added confluence server %s" % server["name"])
cmnds.add("add_confluence_server", handle_add_confluence_server, ["OPER"])
examples.add("add_confluence_server", "add a confluence server", "add_confluence_server FireBreath http://confluence.firebreath.org myuser mypassword")

def handle_del_confluence_server(bot, ievent):
    """ remove a confluence server; syntax: del_confluence_server """
    if len(ievent.args) != 1:
        ievent.reply("syntax: del_confluence_server [server name]")
        return

    serverName = ievent.args[0]
    if "servers" not in cfg.data:
        cfg.data["servers"] = {}
    if serverName in cfg.data["servers"]:
        del cfg.data["servers"][serverName]
        cfg.save()
        ievent.reply("Deleted confluence server %s" % serverName)
    else:
        ievent.reply("Unknown confluence server %s" % serverName)
cmnds.add("del_confluence_server", handle_del_confluence_server, ["OPER"])
examples.add("del_confluence_server", "del a confluence server", "del_confluence_server FireBreath http://confluence.firebreath.org myuser mypassword")

def handle_confluence_enable_server(bot, ievent):
    """ choose the confluence server for lookups in the current channel; syntax: handle_confluence_enable_server [server] """
    if len(ievent.args) != 1:
        ievent.reply("syntax: handle_confluence_enable_server [server]")
        return

    serverName = ievent.args[0]
    if not "servers" in cfg.data or not serverName in cfg.data["servers"]:
        ievent.reply("Unknown server %s" % serverName)
        return

    if not "channels" in cfg.data:
        cfg.data["channels"] = {}

    cfg.data["channels"][ievent.channel] = serverName
    cfg.save()
    ievent.reply("enabled confluence searches from this channel for server %s" % serverName)
cmnds.add("confluence_enable_server", handle_confluence_enable_server, ["OPER"])
examples.add("confluence_enable_server", "enable searching confluence from the channel", "confluence_enable_server confluenceserver")

## confluence-disable command

def handle_confluence_disable(bot, ievent):
    """ disable lookups for confluence in the current channel; syntax: confluence_disable """
    if not "channels" in cfg.data or not ievent.channel in cfg.data["channels"]:
        ievent.reply("Confluence search was not enabled on this channel")
        return

    del cfg.data["channels"][ievent.channel]
    ievent.reply("disabled confluence searching from this channel")
cmnds.add("confluence_disable", handle_confluence_disable, ["OPER"])
examples.add("confluence_disable", "disable lookups for confluence in the current channel", "confluence_disable")

## wiki command

def handle_confluence_search(bot, ievent):

    if "channels" not in cfg.data or ievent.channel not in cfg.data["channels"]:
        ievent.reply("Confluence wiki search not enabled for this channel")
        return

    serverName = cfg.data["channels"][ievent.channel]
    server = cfg.data["servers"][serverName]

    if len(ievent.args) == 0:
        ievent.reply("The wiki is located at %s" % server["url"])
        return
    args = ievent.args
    if args[0][0] == "#":
        maxResults = int(args[0].strip("#"))
        args = args[1:]
    else:
        maxResults = 5

    query = " ".join(args)

    try:
        client, auth = getRpcClient(server)
        results = client.search(auth, query, maxResults)
    except Exception as ex: ievent.reply("an error occured: %s" % str(ex)) ; return

    ievent.reply("Displaying %s result(s) :" % min(maxResults, len(results)))
    for page in results[:maxResults]:
        tinyurl = get_tinyurl(page["url"])
        tinyurl = tinyurl[0] if tinyurl else page["url"]
        ievent.reply('"%s": %s' % (page["title"], tinyurl))

cmnds.add("wiki", handle_confluence_search, ["OPER", "USER", "GUEST"])
examples.add("wiki", "perform a lookup in the selected confluence instance", "wiki #5 some search text")



# -*- coding: utf-8 -*-
"""
    sleekxmpp.xmlstream.matcher.xpath
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Part of SleekXMPP: The Sleek XMPP Library

    :copyright: (c) 2011 Nathanael C. Fritz
    :license: MIT, see LICENSE for more details
"""

from sleekxmpp.xmlstream.stanzabase import ET
from sleekxmpp.xmlstream.matcher.base import MatcherBase


# Flag indicating if the builtin XPath matcher should be used, which
# uses namespaces, or a custom matcher that ignores namespaces.
# Changing this will affect ALL XPath matchers.
IGNORE_NS = False


class MatchXPath(MatcherBase):

    """
    The XPath matcher selects stanzas whose XML contents matches a given
    XPath expression.

    .. warning::

        Using this matcher may not produce expected behavior when using
        attribute selectors. For Python 2.6 and 3.1, the ElementTree
        :meth:`~xml.etree.ElementTree.Element.find()` method does
        not support the use of attribute selectors. If you need to
        support Python 2.6 or 3.1, it might be more useful to use a
        :class:`~sleekxmpp.xmlstream.matcher.stanzapath.StanzaPath` matcher.

    If the value of :data:`IGNORE_NS` is set to ``True``, then XPath
    expressions will be matched without using namespaces.
    """

    def match(self, xml):
        """
        Compare a stanza's XML contents to an XPath expression.

        If the value of :data:`IGNORE_NS` is set to ``True``, then XPath
        expressions will be matched without using namespaces.

        .. warning::

            In Python 2.6 and 3.1 the ElementTree
            :meth:`~xml.etree.ElementTree.Element.find()` method does not
            support attribute selectors in the XPath expression.

        :param xml: The :class:`~sleekxmpp.xmlstream.stanzabase.ElementBase`
                    stanza to compare against.
        """
        if hasattr(xml, 'xml'):
            xml = xml.xml
        x = ET.Element('x')
        x.append(xml)

        if not IGNORE_NS:
            # Use builtin, namespace respecting, XPath matcher.
            if x.find(self._criteria) is not None:
                return True
            return False
        else:
            # Remove namespaces from the XPath expression.
            criteria = []
            for ns_block in self._criteria.split('{'):
                criteria.extend(ns_block.split('}')[-1].split('/'))

            # Walk the XPath expression.
            xml = x
            for tag in criteria:
                if not tag:
                    # Skip empty tag name artifacts from the cleanup phase.
                    continue

                children = [c.tag.split('}')[-1] for c in xml.getchildren()]
                try:
                    index = children.index(tag)
                except ValueError:
                    return False
                xml = xml.getchildren()[index]
            return True



"""
    SleekXMPP: The Sleek XMPP Library
    Copyright (C) 2011  Nathanael C. Fritz
    This file is part of SleekXMPP.

    See the file LICENSE for copying permission.
"""

import logging

from sleekxmpp.stanza import Iq, StreamFeatures
from sleekxmpp.features.feature_bind import stanza
from sleekxmpp.xmlstream import register_stanza_plugin
from sleekxmpp.xmlstream.matcher import *
from sleekxmpp.xmlstream.handler import *
from sleekxmpp.plugins.base import base_plugin


log = logging.getLogger(__name__)


class feature_bind(base_plugin):

    def plugin_init(self):
        self.name = 'Bind Resource'
        self.rfc = '6120'
        self.description = 'Resource Binding Stream Feature'
        self.stanza = stanza

        self.xmpp.register_feature('bind',
                self._handle_bind_resource,
                restart=False,
                order=10000)

        register_stanza_plugin(Iq, stanza.Bind)
        register_stanza_plugin(StreamFeatures, stanza.Bind)

    def _handle_bind_resource(self, features):
        """
        Handle requesting a specific resource.

        Arguments:
            features -- The stream features stanza.
        """
        log.debug("Requesting resource: %s", self.xmpp.boundjid.resource)
        iq = self.xmpp.Iq()
        iq['type'] = 'set'
        iq.enable('bind')
        if self.xmpp.boundjid.resource:
            iq['bind']['resource'] = self.xmpp.boundjid.resource
        response = iq.send(now=True)

        self.xmpp.set_jid(response['bind']['jid'])
        self.xmpp.bound = True

        self.xmpp.features.add('bind')

        log.info("Node set to: %s", self.xmpp.boundjid.full)

        if 'session' not in features['features']:
            log.debug("Established Session")
            self.xmpp.sessionstarted = True
            self.xmpp.session_started_event.set()
            self.xmpp.event("session_start")
